```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement des données - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="ACM_CANINES",index_col=0)

#affichage des caractéristiques
print(D.info())

#récupération des variables actives
DActives = D[['Taille','Velocite','Affection']]
print(DActives)

#récupération des infos - nombre de variables
p = DActives.shape[1]

#nombre d'observations
n = DActives.shape[0]
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 8 entries, Beauceron to Labrador
    Data columns (total 5 columns):
     #   Column     Non-Null Count  Dtype  
    ---  ------     --------------  -----  
     0   Taille     8 non-null      object 
     1   Velocite   8 non-null      object 
     2   Affection  8 non-null      object 
     3   Cote       8 non-null      float64
     4   Fonction   8 non-null      object 
    dtypes: float64(1), object(4)
    memory usage: 384.0+ bytes
    None
                   Taille Velocite Affection
    Chien                                   
    Beauceron    Taille++  Veloc++    Affec+
    Basset         Taill-    Velo-     Affe-
    Berger All   Taille++  Veloc++    Affec+
    Boxer         Taille+   Veloc+    Affec+
    Bull-Dog       Taill-    Velo-    Affec+
    Bull-Mastif  Taille++    Velo-     Affe-
    Caniche        Taill-   Veloc+    Affec+
    Labrador      Taille+   Veloc+    Affec+
    


```python
#codage en 0/1
X = pandas.get_dummies(DActives,prefix='',prefix_sep='')
print(X)

#nombre total de modalités
M = X.shape[1]
```

                 Taill-  Taille+  Taille++  Velo-  Veloc+  Veloc++  Affe-  Affec+
    Chien                                                                        
    Beauceron         0        0         1      0       0        1      0       1
    Basset            1        0         0      1       0        0      1       0
    Berger All        0        0         1      0       0        1      0       1
    Boxer             0        1         0      0       1        0      0       1
    Bull-Dog          1        0         0      1       0        0      0       1
    Bull-Mastif       0        0         1      1       0        0      1       0
    Caniche           1        0         0      0       1        0      0       1
    Labrador          0        1         0      0       1        0      0       1
    


```python
#analyse avec fanalysis
from fanalysis.mca import MCA

#instanciation
acm = MCA(row_labels=DActives.index,var_labels=DActives.columns)
acm.fit(DActives.values)

#propriétés
print(dir(acm))
```

    ['__class__', '__delattr__', '__dict__', '__dir__', '__doc__', '__eq__', '__format__', '__ge__', '__getattribute__', '__getstate__', '__gt__', '__hash__', '__init__', '__init_subclass__', '__le__', '__lt__', '__module__', '__ne__', '__new__', '__reduce__', '__reduce_ex__', '__repr__', '__setattr__', '__setstate__', '__sizeof__', '__str__', '__subclasshook__', '__weakref__', '_binarization', '_compute_stats', '_compute_svd', '_get_param_names', '_get_tags', '_more_tags', 'c_', 'col_contrib_', 'col_coord_', 'col_cos2_', 'col_labels', 'col_labels_', 'col_labels_short_', 'col_labels_short_temp_', 'col_labels_temp_', 'col_topandas', 'eig_', 'fit', 'fit_transform', 'get_params', 'mapping', 'mapping_col', 'mapping_row', 'model_', 'n_', 'n_categories_', 'n_components', 'n_components_', 'n_vars_', 'plot_col_contrib', 'plot_col_cos2', 'plot_eigenvalues', 'plot_row_contrib', 'plot_row_cos2', 'prefixes_', 'r_', 'row_contrib_', 'row_coord_', 'row_cos2_', 'row_labels', 'row_labels_', 'row_topandas', 'set_params', 'stats', 'transform', 'var_labels']
    


```python
#nombre max de facteurs
Hmax = M-p

#numpy
import numpy

#valeurs propres
print(pandas.DataFrame(numpy.transpose(acm.eig_),columns=['Val.P','%','Cumul %'],index=range(1,Hmax+1)))
```

          Val.P          %     Cumul %
    1  0.708031  42.481875   42.481875
    2  0.591489  35.489362   77.971237
    3  0.261992  15.719509   93.690746
    4  0.069747   4.184791   97.875538
    5  0.035408   2.124462  100.000000
    


```python
#somme des carrés des valeurs propres
print(numpy.sum(acm.eig_[0]**2))
```

    0.9259259259259274
    


```python
#librairie graphique
import matplotlib.pyplot as plt

#éboulis des v.p.
fix,ax = plt.subplots(figsize=(5,5))
ax.plot(range(1,Hmax+1),acm.eig_[0],".-")
ax.set_xlabel("Nb. facteurs")
ax.set_ylabel("Val. propres")
plt.title("Eboulis des valeurs propres")

#seuil - Règle de Kaiser
ax.plot([1,Hmax],[1/p,1/p],"r--",linewidth=1)
plt.show()
```


![png](output_6_0.png)



```python
#récupérer les valeurs propres supérieur à (1/p)
lambada = acm.eig_[0][acm.eig_[0]>1/p]
print(lambada)
```

    [0.70803126 0.59148936]
    


```python
#appliquer la correction
lambada_prim = ((p/(p-1))*(lambada-1/p))**2
print(lambada_prim)
```

    [0.3158967  0.14995021]
    


```python
#faire la somme
S_prim = numpy.sum(lambada_prim)
print(S_prim)

#et produire les pourcentages
percent_prim = lambada_prim/S_prim*100

#affichage
print(pandas.DataFrame(numpy.transpose(numpy.array([lambada_prim,percent_prim,numpy.cumsum(percent_prim)])),columns=['Val.P','%','Cumul %'],index=range(1,3)))
```

    0.4658469098168333
          Val.P          %     Cumul %
    1  0.315897  67.811269   67.811269
    2  0.149950  32.188731  100.000000
    


```python
#somme corrigée de Greenacre
S2nd = p/(p-1)*(numpy.sum(acm.eig_[0]**2)-(M-p)/(p**2))
print(S2nd)
```

    0.5555555555555577
    


```python
#pourcentage corrigé Greenacre
percent_2nd = lambada_prim/S2nd

#affichage
print(pandas.DataFrame(numpy.transpose(numpy.array([lambada_prim,percent_2nd,numpy.cumsum(percent_2nd)])),columns=['Val.P','%','Cumul %'],index=range(1,3)))
```

          Val.P         %   Cumul %
    1  0.315897  0.568614  0.568614
    2  0.149950  0.269910  0.838524
    


```python
#représentation dans le plan
acm.mapping(num_x_axis=1,num_y_axis=2)
```


![png](output_12_0.png)



```python
#affichage des coordonnées
print(pandas.DataFrame(acm.col_coord_[:,:2],index=X.columns,columns=['Coord.F1','Coord.F2']))
```

                  Coord.F1  Coord.F2
    Taill-    4.558738e-01  0.788128
    Taille+  -1.367621e+00  0.500758
    Taille++  4.558738e-01 -1.121966
    Velo-     1.081461e+00  0.554740
    Veloc+   -1.081461e+00  0.554740
    Veloc++   7.058942e-15 -1.664220
    Affe-     1.367621e+00  0.500758
    Affec+   -4.558738e-01 -0.166919
    


```python
#représentation dans le plan
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Représentation des points-modalités")

for i in range(X.shape[1]):
    ax.text(acm.col_coord_[i,0],acm.col_coord_[i,1],X.columns[i],color='dodgerblue')
          
plt.show()
```


![png](output_14_0.png)



```python
#affichage des contributions
print(pandas.DataFrame(acm.col_contrib_[:,:2],index=X.columns,columns=['Contrib.F1','Contrib.F2']))
```

                Contrib.F1  Contrib.F2
    Taill-    3.668993e+00   13.126725
    Taille+   2.201396e+01    3.532865
    Taille++  3.668993e+00   26.602515
    Velo-     2.064806e+01    6.503426
    Veloc+    2.064806e+01    6.503426
    Veloc++   5.864697e-28   39.020556
    Affe-     2.201396e+01    3.532865
    Affec+    7.337985e+00    1.177622
    


```python
#ou bien graphiquement
acm.plot_col_contrib(num_axis=1)
```


![png](output_16_0.png)



```python
#affichage des COS2
print(pandas.DataFrame(acm.col_cos2_[:,:2],index=X.columns,columns=['Cos2.F1','Cos2.F2']))
```

                   Cos2.F1   Cos2.F2
    Taill-    1.246926e-01  0.372687
    Taille+   6.234628e-01  0.083586
    Taille++  1.246926e-01  0.755285
    Velo-     7.017346e-01  0.184642
    Veloc+    7.017346e-01  0.184642
    Veloc++   1.660956e-29  0.923210
    Affe-     6.234628e-01  0.083586
    Affec+    6.234628e-01  0.083586
    


```python
#affichage des COS2 cumulés
print(pandas.DataFrame(numpy.cumsum(acm.col_cos2_[:,:2],axis=1),index=X.columns,columns=['Cos2.F1','Cum(Cos2).F2']))
```

                   Cos2.F1  Cum(Cos2).F2
    Taill-    1.246926e-01      0.497380
    Taille+   6.234628e-01      0.707049
    Taille++  1.246926e-01      0.879978
    Velo-     7.017346e-01      0.886377
    Veloc+    7.017346e-01      0.886377
    Veloc++   1.660956e-29      0.923210
    Affe-     6.234628e-01      0.707049
    Affec+    6.234628e-01      0.707049
    


```python
dir(acm)
```




    ['__class__',
     '__delattr__',
     '__dict__',
     '__dir__',
     '__doc__',
     '__eq__',
     '__format__',
     '__ge__',
     '__getattribute__',
     '__getstate__',
     '__gt__',
     '__hash__',
     '__init__',
     '__init_subclass__',
     '__le__',
     '__lt__',
     '__module__',
     '__ne__',
     '__new__',
     '__reduce__',
     '__reduce_ex__',
     '__repr__',
     '__setattr__',
     '__setstate__',
     '__sizeof__',
     '__str__',
     '__subclasshook__',
     '__weakref__',
     '_binarization',
     '_compute_stats',
     '_compute_svd',
     '_get_param_names',
     '_get_tags',
     '_more_tags',
     'c_',
     'col_contrib_',
     'col_coord_',
     'col_cos2_',
     'col_labels',
     'col_labels_',
     'col_labels_short_',
     'col_labels_short_temp_',
     'col_labels_temp_',
     'col_topandas',
     'eig_',
     'fit',
     'fit_transform',
     'get_params',
     'mapping',
     'mapping_col',
     'mapping_row',
     'model_',
     'n_',
     'n_categories_',
     'n_components',
     'n_components_',
     'n_vars_',
     'plot_col_contrib',
     'plot_col_cos2',
     'plot_eigenvalues',
     'plot_row_contrib',
     'plot_row_cos2',
     'prefixes_',
     'r_',
     'row_contrib_',
     'row_coord_',
     'row_cos2_',
     'row_labels',
     'row_labels_',
     'row_topandas',
     'set_params',
     'stats',
     'transform',
     'var_labels']




```python
#effectifs par modalité
print(acm.c_)
```

    [[3. 2. 3. 3. 3. 2. 2. 6.]]
    


```python
#calul des valeurs test par modalité - 1er facteur
vtest = acm.col_coord_[:,0]*numpy.sqrt(((n-1)*acm.c_[0])/(n-acm.c_[0]))
print(pandas.DataFrame(vtest,index=X.columns))
```

                         0
    Taill-    9.342633e-01
    Taille+  -2.089076e+00
    Taille++  9.342633e-01
    Velo-     2.216335e+00
    Veloc+   -2.216335e+00
    Veloc++   1.078271e-14
    Affe-     2.089076e+00
    Affec+   -2.089076e+00
    


```python
#récupération des contribution sous une forme exploitable - 2 facteurs
contrib12 = pandas.DataFrame(acm.col_contrib_[:,:2],index=X.columns)
print(contrib12)
```

                         0          1
    Taill-    3.668993e+00  13.126725
    Taille+   2.201396e+01   3.532865
    Taille++  3.668993e+00  26.602515
    Velo-     2.064806e+01   6.503426
    Veloc+    2.064806e+01   6.503426
    Veloc++   5.864697e-28  39.020556
    Affe-     2.201396e+01   3.532865
    Affec+    7.337985e+00   1.177622
    


```python
#**calcul des carrés des rapports de corrélation par variable sur les 2 facteurs**

#structure pour les eta2
eta2 = numpy.zeros((3,2))

#pour chaque facteur
for h in range(2):
    #pour chaque variable
    for j in range(3):
        eta2[j,h] = (p*acm.eig_[0][h]*numpy.sum(contrib12.loc[numpy.unique(D.iloc[:,j])][h]))/100

#affichage
print(pandas.DataFrame(eta2,index=D.columns[:3],columns=['F1','F2']))
```

                     F1        F2
    Taille     0.623463  0.767672
    Velocite   0.877168  0.923210
    Affection  0.623463  0.083586
    


```python
#réprésentation des variables dans le plan
fix,ax = plt.subplots(figsize=(5,5))
ax.axis([0,+1,0,+1])
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carrés rapports de corrélation")

for i in range(3):
    ax.text(eta2[i,0],eta2[i,1],D.columns[i],color='darkcyan')
          
plt.show()

```


![png](output_24_0.png)



```python
#coordonnées des individus dans le plan
print(pandas.DataFrame(acm.row_coord_[:,:2],index=X.index,columns=['Coord.F1','Coord.F2']))
```

                     Coord.F1  Coord.F2
    Chien                              
    Beauceron    4.924471e-15 -1.279924
    Basset       1.150779e+00  0.799057
    Berger All   4.763557e-15 -1.279924
    Boxer       -1.150779e+00  0.385124
    Bull-Dog     4.284137e-01  0.509675
    Bull-Mastif  1.150779e+00 -0.028809
    Caniche     -4.284137e-01  0.509675
    Labrador    -1.150779e+00  0.385124
    


```python
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Représentation des individus")

for i in range(X.shape[0]):
    ax.text(acm.row_coord_[i,0],acm.row_coord_[i,1],X.index[i],color='firebrick')
          
plt.show()
```


![png](output_26_0.png)



```python
#afficher les contributions
print(pandas.DataFrame(acm.row_contrib_[:,:2],index=X.index,columns=['Contrib.F1','Contrib.F2']))
```

                   Contrib.F1  Contrib.F2
    Chien                                
    Beauceron    4.281311e-28   34.620365
    Basset       2.337985e+01   13.493329
    Berger All   4.006086e-28   34.620365
    Boxer        2.337985e+01    3.134479
    Bull-Dog     3.240293e+00    5.489722
    Bull-Mastif  2.337985e+01    0.017539
    Caniche      3.240293e+00    5.489722
    Labrador     2.337985e+01    3.134479
    


```python
#ou bien graphiquement
acm.plot_row_contrib(num_axis=1)
```


![png](output_28_0.png)



```python
#affichage des COS2
print(pandas.DataFrame(acm.row_cos2_[:,:2],index=X.index,columns=['Cos2.F1','Cos2.F2']))
```

                      Cos2.F1   Cos2.F2
    Chien                              
    Beauceron    1.455025e-29  0.982924
    Basset       6.272969e-01  0.302444
    Berger All   1.361488e-29  0.982924
    Boxer        7.945760e-01  0.088993
    Bull-Dog     1.501677e-01  0.212538
    Bull-Mastif  6.272969e-01  0.000393
    Caniche      1.501677e-01  0.212538
    Labrador     7.945760e-01  0.088993
    


```python
#affichage des COS2 cumulés
print(pandas.DataFrame(numpy.cumsum(acm.row_cos2_[:,:2],axis=1),index=X.index,columns=['Cos2.F1','Cum(Cos2).F2']))
```

                      Cos2.F1  Cum(Cos2).F2
    Chien                                  
    Beauceron    1.455025e-29      0.982924
    Basset       6.272969e-01      0.929741
    Berger All   1.361488e-29      0.982924
    Boxer        7.945760e-01      0.883569
    Bull-Dog     1.501677e-01      0.362706
    Bull-Mastif  6.272969e-01      0.627690
    Caniche      1.501677e-01      0.362706
    Labrador     7.945760e-01      0.883569
    


```python
#profil du Basset
profil_Basset = X.loc["Basset"]/p
print(profil_Basset)
```

    Taill-      0.333333
    Taille+     0.000000
    Taille++    0.000000
    Velo-       0.333333
    Veloc+      0.000000
    Veloc++     0.000000
    Affe-       0.333333
    Affec+      0.000000
    Name: Basset, dtype: float64
    


```python
#rappel coordonnées factorielles des modalités - 1er facteur
print(acm.col_coord_[:,0])
```

    [ 4.55873798e-01 -1.36762139e+00  4.55873798e-01  1.08146090e+00
     -1.08146090e+00  7.05894227e-15  1.36762139e+00 -4.55873798e-01]
    


```python
#coordonnées du Basset
coord_Basset = (1/numpy.sqrt(acm.eig_[0][0]))*numpy.dot(profil_Basset,acm.col_coord_[:,0])
print(coord_Basset)
```

    1.150779460602135
    


```python
#vérification
print(pandas.DataFrame(acm.row_coord_[:,0],index=X.index).loc['Basset'])
```

    0    1.150779
    Name: Basset, dtype: float64
    


```python
#représentation simultanée - relation quasi-barycentrique
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Représentation simultanée - 1")

for i in range(X.shape[1]):
    ax.text(acm.col_coord_[i,0],acm.col_coord_[i,1],X.columns[i],color='dodgerblue')
    
for i in range(X.shape[0]):
    ax.text(acm.row_coord_[i,0],acm.row_coord_[i,1],X.index[i],color='firebrick')
          
plt.show()
```


![png](output_35_0.png)



```python
#données temporaires liant Taille et Facteur (1,2)
temp = pandas.DataFrame(acm.row_coord_[:,:2],columns=['F1','F2'],index=X.index)
temp['Taille'] = D.Taille
print(temp)
```

                           F1        F2    Taille
    Chien                                        
    Beauceron    4.924471e-15 -1.279924  Taille++
    Basset       1.150779e+00  0.799057    Taill-
    Berger All   4.763557e-15 -1.279924  Taille++
    Boxer       -1.150779e+00  0.385124   Taille+
    Bull-Dog     4.284137e-01  0.509675    Taill-
    Bull-Mastif  1.150779e+00 -0.028809  Taille++
    Caniche     -4.284137e-01  0.509675    Taill-
    Labrador    -1.150779e+00  0.385124   Taille+
    


```python
#moyennes conditionnelles
print(pandas.pivot_table(temp,values=['F1','F2'],index='Taille',aggfunc='mean'))
```

                    F1        F2
    Taille                      
    Taill-    0.383593  0.606136
    Taille+  -1.150779  0.385124
    Taille++  0.383593 -0.862886
    


```python
#coordonnées des modalités - relations barycentriques
coord_moda_2 = numpy.array([acm.col_coord_[:,0]*numpy.sqrt(acm.eig_[0][0]),acm.col_coord_[:,1]*numpy.sqrt(acm.eig_[0][1])])
print(pandas.DataFrame(numpy.transpose(coord_moda_2),index=X.columns,columns=['G_prim_1','G_prim_2']))
```

                  G_prim_1  G_prim_2
    Taill-    3.835932e-01  0.606136
    Taille+  -1.150779e+00  0.385124
    Taille++  3.835932e-01 -0.862886
    Velo-     9.099909e-01  0.426641
    Veloc+   -9.099909e-01  0.426641
    Veloc++   5.939718e-15 -1.279924
    Affe-     1.150779e+00  0.385124
    Affec+   -3.835932e-01 -0.128375
    


```python
#représentation simultanée - relation barycentrique
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Représentation simultanée - 2")

for i in range(X.shape[1]):
    ax.text(coord_moda_2[0,i],coord_moda_2[1,i],X.columns[i],color='dodgerblue')
    
for i in range(X.shape[0]):
    ax.text(acm.row_coord_[i,0],acm.row_coord_[i,1],X.index[i],color='firebrick')
          
plt.show()
```


![png](output_39_0.png)



```python
#rappel coordonnées factorielles des individus
print(pandas.DataFrame(acm.row_coord_[:,:2],index=X.index))
```

                            0         1
    Chien                              
    Beauceron    4.924471e-15 -1.279924
    Basset       1.150779e+00  0.799057
    Berger All   4.763557e-15 -1.279924
    Boxer       -1.150779e+00  0.385124
    Bull-Dog     4.284137e-01  0.509675
    Bull-Mastif  1.150779e+00 -0.028809
    Caniche     -4.284137e-01  0.509675
    Labrador    -1.150779e+00  0.385124
    


```python
#fréquences des modalités
freq_moda = acm.c_[0]/n
print(freq_moda)
```

    [0.375 0.25  0.375 0.375 0.375 0.25  0.25  0.75 ]
    


```python
#rappel des coordonnées des modalités
print(pandas.DataFrame(acm.col_coord_[:,:2],index=X.columns))
```

                         0         1
    Taill-    4.558738e-01  0.788128
    Taille+  -1.367621e+00  0.500758
    Taille++  4.558738e-01 -1.121966
    Velo-     1.081461e+00  0.554740
    Veloc+   -1.081461e+00  0.554740
    Veloc++   7.058942e-15 -1.664220
    Affe-     1.367621e+00  0.500758
    Affec+   -4.558738e-01 -0.166919
    


```python
#tableau des indicatrices pour vérification
print(X)
```

                 Taill-  Taille+  Taille++  Velo-  Veloc+  Veloc++  Affe-  Affec+
    Chien                                                                        
    Beauceron         0        0         1      0       0        1      0       1
    Basset            1        0         0      1       0        0      1       0
    Berger All        0        0         1      0       0        1      0       1
    Boxer             0        1         0      0       1        0      0       1
    Bull-Dog          1        0         0      1       0        0      0       1
    Bull-Mastif       0        0         1      1       0        0      1       0
    Caniche           1        0         0      0       1        0      0       1
    Labrador          0        1         0      0       1        0      0       1
    


```python
#tableau reconstitué
temp = acm.row_coord_[:,:2]/numpy.sqrt(acm.eig_[0][:2])
temp = 1+numpy.dot(temp,numpy.transpose(acm.col_coord_[:,:2]))
Xhat = numpy.around(temp*freq_moda,2)
print(pandas.DataFrame(Xhat,index=X.index,columns=X.columns))
```

                 Taill-  Taille+  Taille++  Velo-  Veloc+  Veloc++  Affe-  Affec+
    Chien                                                                        
    Beauceron     -0.12     0.04      1.08   0.03    0.03     0.94   0.04    0.96
    Basset         0.92    -0.09      0.17   1.15    0.04    -0.18   0.85    0.15
    Berger All    -0.12     0.04      1.08   0.03    0.03     0.94   0.04    0.96
    Boxer          0.29     0.78     -0.07  -0.08    1.03     0.04  -0.15    1.15
    Bull-Dog       0.66     0.16      0.18   0.72    0.31    -0.03   0.51    0.49
    Bull-Mastif    0.60    -0.22      0.62   0.92   -0.19     0.27   0.71    0.29
    Caniche        0.48     0.51      0.01   0.31    0.72    -0.03   0.16    0.84
    Labrador       0.29     0.78     -0.07  -0.08    1.03     0.04  -0.15    1.15
    


```python
#estimation booléenne sur les 2 premiers facteurs
print(pandas.DataFrame((Xhat > (p/M)).astype(int),index=X.index,columns=X.columns))
```

                 Taill-  Taille+  Taille++  Velo-  Veloc+  Veloc++  Affe-  Affec+
    Chien                                                                        
    Beauceron         0        0         1      0       0        1      0       1
    Basset            1        0         0      1       0        0      1       0
    Berger All        0        0         1      0       0        1      0       1
    Boxer             0        1         0      0       1        0      0       1
    Bull-Dog          1        0         0      1       0        0      1       1
    Bull-Mastif       1        0         1      1       0        0      1       0
    Caniche           1        1         0      0       1        0      0       1
    Labrador          0        1         0      0       1        0      0       1
    


```python
#comparaison
temp = (Xhat > (p/M)).astype(int)
print(X.values-temp)
```

    [[ 0  0  0  0  0  0  0  0]
     [ 0  0  0  0  0  0  0  0]
     [ 0  0  0  0  0  0  0  0]
     [ 0  0  0  0  0  0  0  0]
     [ 0  0  0  0  0  0 -1  0]
     [-1  0  0  0  0  0  0  0]
     [ 0 -1  0  0  0  0  0  0]
     [ 0  0  0  0  0  0  0  0]]
    


```python
#chargement de l'individu supplémentaire
DSupp = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="ACM_CANINES_SUPP",index_col=0)
print(DSupp)
```

               Taille Velocite Affection
    Chien                               
    Levrier  Taille++  Veloc++     Affe-
    


```python
#rappel des coordonnées des modalités
print(pandas.DataFrame(acm.col_coord_[:,:2],index=X.columns))
```

                         0         1
    Taill-    4.558738e-01  0.788128
    Taille+  -1.367621e+00  0.500758
    Taille++  4.558738e-01 -1.121966
    Velo-     1.081461e+00  0.554740
    Veloc+   -1.081461e+00  0.554740
    Veloc++   7.058942e-15 -1.664220
    Affe-     1.367621e+00  0.500758
    Affec+   -4.558738e-01 -0.166919
    


```python
#rappel des valeurs propres
print(acm.eig_[0][:2])
```

    [0.70803126 0.59148936]
    


```python
#vecteur pour individu supplémentaire
XSupp = numpy.zeros(M)

#codage 0/1 de l'individu supp
for v in DSupp.loc['Levrier'].values:
    XSupp[numpy.where(X.columns==v)] = 1

#profil
XSupp = XSupp / p
print(XSupp)
```

    [0.         0.         0.33333333 0.         0.         0.33333333
     0.33333333 0.        ]
    


```python
#projection
coord_supp = numpy.dot(numpy.reshape(XSupp,(1,M)),acm.col_coord_[:,:2])
coord_supp = coord_supp / numpy.sqrt(acm.eig_[0][:2])
print(coord_supp)
```

    [[ 0.72236576 -0.99054221]]
    


```python
#positionnement dans le plan factoriel
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Individu supplémentaire")
#individus actifs
for i in range(X.shape[0]):
    ax.text(acm.row_coord_[i,0],acm.row_coord_[i,1],X.index[i],color='firebrick')
#individu supplémentaire
ax.text(coord_supp[0][0],coord_supp[0][1],'Levrier',color='magenta',fontsize=12)
          
plt.show()
```


![png](output_52_0.png)



```python
#variables illustratives
print(D[['Cote','Fonction']])
```

                 Cote   Fonction
    Chien                       
    Beauceron     2.0    utilite
    Basset        4.5     chasse
    Berger All    2.5    utilite
    Boxer         3.0  compagnie
    Bull-Dog      1.5  compagnie
    Bull-Mastif   1.0    utilite
    Caniche       4.0  compagnie
    Labrador      3.5     chasse
    


```python
#corrélation de Cote avec les 2 premiers facteurs
corr_cote = numpy.corrcoef(D.Cote,acm.row_coord_[:,:2],rowvar=False)[0,1:]
print(corr_cote)
```

    [-0.28809902  0.44175308]
    


```python
#positionnement dans un "cercle des corrélations"
fig, ax = plt.subplots(figsize=(5,5))
ax.axis([-1,+1,-1,+1])
ax.plot([-1,+1],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-1,+1],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Cercle des corrélations")

#variables illustratives
ax.text(corr_cote[0],corr_cote[1],'Cote',color='brown')
ax.arrow(0,0,corr_cote[0]+0.1,corr_cote[1]-0.1,color='r',head_width=0.05)

#cercle
from matplotlib.patches import Ellipse
ellipse = Ellipse((0,0),width=2,height=2,facecolor='none',edgecolor='silver',linestyle='--')
ax.add_patch(ellipse)
    
plt.show()
```


![png](output_55_0.png)



```python
#structure temporaire réunissant les coord. fact. (1,2) et fonction
df = pandas.DataFrame(acm.row_coord_[:,:2],index=X.index,columns=['F1','F2'])
df['Fonction'] = D.Fonction
print(df)
```

                           F1        F2   Fonction
    Chien                                         
    Beauceron    4.924471e-15 -1.279924    utilite
    Basset       1.150779e+00  0.799057     chasse
    Berger All   4.763557e-15 -1.279924    utilite
    Boxer       -1.150779e+00  0.385124  compagnie
    Bull-Dog     4.284137e-01  0.509675  compagnie
    Bull-Mastif  1.150779e+00 -0.028809    utilite
    Caniche     -4.284137e-01  0.509675  compagnie
    Labrador    -1.150779e+00  0.385124     chasse
    


```python
#calcul des moyennes conditionnelles
GBar = pandas.pivot_table(df,values=['F1','F2'],index="Fonction",aggfunc='mean')
print(GBar)
```

                         F1        F2
    Fonction                         
    chasse    -2.109424e-15  0.592091
    compagnie -3.835932e-01  0.468158
    utilite    3.835932e-01 -0.862886
    


```python
#coordonnées factorielles des modalités - Solution 1
Gv1 = GBar.values/numpy.sqrt(acm.eig_[0][:2])
Gv1 = pandas.DataFrame(Gv1,index=GBar.index,columns=GBar.columns)
print(Gv1)
```

                         F1        F2
    Fonction                         
    chasse    -2.506903e-15  0.769866
    compagnie -4.558738e-01  0.608722
    utilite    4.558738e-01 -1.121966
    


```python
#représentation dans le plan
fix,ax = plt.subplots(figsize=(7,7))
ax.axis([-2,+2,-2,+2])
ax.plot([-2,+2],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-2,+2],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Points-modalités supplémentaires")

#modalités actives
for i in range(X.shape[1]):
    ax.text(acm.col_coord_[i,0],acm.col_coord_[i,1],X.columns[i],color='dodgerblue')
    
#modalités illustratives de la variable "fonction"
for i in Gv1.index:
    ax.text(Gv1.loc[i][0],Gv1.loc[i][1],i,color='blue')
          
plt.show()
```


![png](output_59_0.png)

