```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement des données - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="DATA_ACP_ACTIF",index_col=0)

#affichage des caractéristiques
print(D.info())
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 18 entries, Alfasud TI to Lada 1300
    Data columns (total 6 columns):
     #   Column  Non-Null Count  Dtype
    ---  ------  --------------  -----
     0   CYL     18 non-null     int64
     1   PUISS   18 non-null     int64
     2   LONG    18 non-null     int64
     3   LARG    18 non-null     int64
     4   POIDS   18 non-null     int64
     5   VMAX    18 non-null     int64
    dtypes: int64(6)
    memory usage: 1008.0+ bytes
    None
    


```python
#affichage des varleurs
print(D)
```

                      CYL  PUISS  LONG  LARG  POIDS  VMAX
    Modele                                               
    Alfasud TI       1350     79   393   161    870   165
    Audi 100         1588     85   468   177   1110   160
    Simca 1300       1294     68   424   168   1050   152
    Citroen GS Club  1222     59   412   161    930   151
    Fiat 132         1585     98   439   164   1105   165
    Lancia Beta      1297     82   429   169   1080   160
    Peugeot 504      1796     79   449   169   1160   154
    Renault 16 TL    1565     55   424   163   1010   140
    Renault 30       2664    128   452   173   1320   180
    Toyota Corolla   1166     55   399   157    815   140
    Alfetta 1.66     1570    109   428   162   1060   175
    Princess 1800    1798     82   445   172   1160   158
    Datsun 200L      1998    115   469   169   1370   160
    Taunus 2000      1993     98   438   170   1080   167
    Rancho           1442     80   431   166   1129   144
    Mazda 9295       1769     83   440   165   1095   165
    Opel Rekord      1979    100   459   173   1120   173
    Lada 1300        1294     68   404   161    955   140
    


```python
%matplotlib inline

#importer la librairie graphique
import matplotlib.pyplot as plt

#préparer le graphique
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(D.CYL,D.PUISS,"wo")
ax.axis([1000,3000,50,140])
ax.set_xlabel("CYL")
ax.set_ylabel("PUISS")

#ajouter les labels des véhicules
for v in D.index:
    ax.text(D.CYL[v],D.PUISS[v],v)
    
#faire afficher
plt.show()
```


![png](output_3_0.png)



```python
#afficher les deux variables en triant selon CYL
Dbis = D.sort_values(by="CYL",ascending=True)[['CYL','PUISS']]
print(Dbis)
```

                      CYL  PUISS
    Modele                      
    Toyota Corolla   1166     55
    Citroen GS Club  1222     59
    Lada 1300        1294     68
    Simca 1300       1294     68
    Lancia Beta      1297     82
    Alfasud TI       1350     79
    Rancho           1442     80
    Renault 16 TL    1565     55
    Alfetta 1.66     1570    109
    Fiat 132         1585     98
    Audi 100         1588     85
    Mazda 9295       1769     83
    Peugeot 504      1796     79
    Princess 1800    1798     82
    Opel Rekord      1979    100
    Taunus 2000      1993     98
    Datsun 200L      1998    115
    Renault 30       2664    128
    


```python
#librairie graphique
import seaborn as sns

#pairplot
sns.pairplot(D)
```




    <seaborn.axisgrid.PairGrid at 0x1e14b0a6748>




![png](output_5_1.png)



```python
#librairie numpy
import numpy

#distance entre l'Audi et la Fiat
numpy.sqrt(numpy.sum((Dbis.loc['Audi 100']-Dbis.loc['Fiat 132'])**2))
```




    13.341664064126334




```python
#distance entre l'Audi et la Mazda
numpy.sqrt(numpy.sum((Dbis.loc['Audi 100']-Dbis.loc['Mazda 9295'])**2))
```




    181.01104938649465




```python
#matrice X
X = Dbis.values

#écarts-type
sigmas = numpy.std(X,axis=0,ddof=0)
print(sigmas)
```

    [363.39449027  19.80218531]
    


```python
#distance pondérée entre l'Audi et la Fiat
numpy.sqrt(numpy.sum((1/sigmas**2)*(Dbis.loc['Audi 100']-Dbis.loc['Fiat 132'])**2))
```




    0.6565451047185447




```python
#distance entre l'Audi et la Mazda
numpy.sqrt(numpy.sum((1/sigmas**2)*(Dbis.loc['Audi 100']-Dbis.loc['Mazda 9295'])**2))
```




    0.5082182304292723




```python
#nombre d'observations
n = X.shape[0]
print(n)

#inertie version 1 -- distances entre paires d'individus
Ip_v1 = 0

#double boucle
for i1 in range(n):
    for i2 in range(n):
        Ip_v1 = Ip_v1 + numpy.sum((X[i1,:]-X[i2,:])**2)
        
#moyenne des écarts entre paires d'individus
Ip_v1 = (1/(2*n**2)) * Ip_v1
print("Inertie, Approche 1 = %.2f" % (Ip_v1))
```

    18
    Inertie, Approche 1 = 132447.68
    


```python
#moyennes des variables -- Coordonnées de G
moyennes = numpy.mean(X,axis=0)
print(moyennes)
```

    [1631.66666667   84.61111111]
    


```python
#graphique des points avec le barycentre G
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(D.CYL,D.PUISS,color='xkcd:light blue',marker='o',linestyle='None')
ax.axis([1000,3000,50,140])
ax.set_xlabel("CYL")
ax.set_ylabel("PUISS")
#ajouter des traits pointillés entre les points et le barycentre
for i in range(D.shape[0]):
    ax.plot([moyennes[0],D.CYL[i]],[moyennes[1],D.PUISS[i]],color='silver',linestyle='dashed')
#barycentre G
ax.text(moyennes[0],moyennes[1],"G",fontsize=14)
#faire afficher
plt.show()
```


![png](output_13_0.png)



```python
#vecteur des moyennes
print(moyennes)

#inertie version 2 - écarts au barycentre
Ip_v2 = numpy.mean(numpy.apply_along_axis(func1d=lambda x: numpy.sum((x-moyennes)**2),axis=1,arr=X))
print("Inertie, Approche 2 = %.2f" % (Ip_v2))
```

    [1631.66666667   84.61111111]
    Inertie, Approche 2 = 132447.68
    


```python
#écarts-type par variable
print(sigmas)

#vérification -- somme des variances
print("Somme des variances = %.2f" % (numpy.sum(sigmas**2)))
```

    [363.39449027  19.80218531]
    Somme des variances = 132447.68
    


```python
#données centrées et réduites
Z = (X - moyennes)/sigmas
print(pandas.DataFrame(Z,index=Dbis.index))

#vérification moyennes
print(numpy.mean(Z,axis=0))

#vérification écarts-type
print(numpy.std(Z,axis=0,ddof=0))

#outil pour l'ACP
from sklearn.decomposition import PCA
acp = PCA()

#coordonnées factorielles
coord = acp.fit_transform(Z)

#afficher les nouvelles coordonnées
print(pandas.DataFrame(coord,index=Dbis.index))
```

                            0         1
    Modele                             
    Toyota Corolla  -1.281436 -1.495346
    Citroen GS Club -1.127333 -1.293348
    Lada 1300       -0.929201 -0.838852
    Simca 1300      -0.929201 -0.838852
    Lancia Beta     -0.920946 -0.131860
    Alfasud TI      -0.775099 -0.283358
    Rancho          -0.521930 -0.232859
    Renault 16 TL   -0.183455 -1.495346
    Alfetta 1.66    -0.169696  1.231626
    Fiat 132        -0.128419  0.676132
    Audi 100        -0.120163  0.019639
    Mazda 9295       0.377918 -0.081360
    Peugeot 504      0.452217 -0.283358
    Princess 1800    0.457721 -0.131860
    Opel Rekord      0.955802  0.777131
    Taunus 2000      0.994328  0.676132
    Datsun 200L      1.008087  1.534623
    Renault 30       2.840806  2.191116
    [-1.72701359e-16 -1.48029737e-16]
    [1. 1.]
                            0         1
    Modele                             
    Toyota Corolla  -1.963481 -0.151257
    Citroen GS Club -1.711680 -0.117390
    Lada 1300       -1.250203  0.063886
    Simca 1300      -1.250203  0.063886
    Lancia Beta     -0.744446  0.557968
    Alfasud TI      -0.748442  0.347713
    Rancho          -0.533717  0.204405
    Renault 16 TL   -1.187092 -0.927647
    Alfetta 1.66     0.750898  0.990885
    Fiat 132         0.387292  0.568903
    Audi 100        -0.071082  0.098855
    Mazda 9295       0.209698 -0.324759
    Peugeot 504      0.119402 -0.520131
    Princess 1800    0.230419 -0.416897
    Opel Rekord      1.225369 -0.126340
    Taunus 2000      1.181194 -0.224999
    Datsun 200L      1.797968  0.372317
    Renault 30       3.558106 -0.459400
    


```python
#position des véhicules dans le repère factoriel
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(coord[:,0],coord[:,1],"wo")
ax.axis([-4,+4,-4,+4])
ax.plot([-4,+4],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-4,+4],color='silver',linestyle='--')
ax.set_xlabel("Comp.1 (89.83%)")
ax.set_ylabel("Comp.2 (10.17%)")

#ajouter les labels des véhicules
for i in range(n):
    ax.text(coord[i,0],coord[i,1],Dbis.index[i])
    
#faire afficher
plt.show()
```


![png](output_17_0.png)



```python
#calculer les lambda
lambada = numpy.mean(numpy.power(coord,2),axis=0)
print(lambada)
```

    [1.79662771 0.20337229]
    


```python
#part d'inertie expliquée par les axes
print(lambada/numpy.sum(lambada))
```

    [0.89831386 0.10168614]
    


```python
#somme des lambda = inertie totale
print("Somme des lambda = %f" % (numpy.sum(lambada)))
```

    Somme des lambda = 2.000000
    


```python
#data frame des données centrées et réduites
DZ = pandas.DataFrame(Z,index=Dbis.index)

#data frame des coordonnées factorielles
DC = pandas.DataFrame(coord,index=Dbis.index)

#(distance euclidienne)^2 entre les Toyota Corolla et Citroen GS
#données originelles, centrées et réduites
print(numpy.sum((DZ.loc['Toyota Corolla']-DZ.loc['Citroen GS Club'])**2))

#(distance euclidienne)^2 en en prenant en compte que la première composante
print(numpy.sum((DC.loc['Toyota Corolla'][0]-DC.loc['Citroen GS Club'][0])**2))
```

    0.06455073567278283
    0.06340375071089865
    


```python
#distance initiale entre Audi 100 et Fiat 132
print(numpy.sum((DZ.loc['Audi 100']-DZ.loc['Fiat 132'])**2))

#distance approchée (Facteur F1)
print(numpy.sum((DC.loc['Audi 100'][0]-DC.loc['Fiat 132'][0])**2))
```

    0.4310514745298847
    0.2101060634608986
    


```python
#distance si on prend en compte (F1, F2)
print(numpy.sum((DC.loc['Audi 100']-DC.loc['Fiat 132'])**2))
```

    0.43105147452988485
    


```python
#former la matrice X avec (p=6) variables maintenant
X = D.values

#calculer la matrice de covariance
#rowvar = False pour dire que les variables sont organisées en colonnes
#ddof = 0 pour utiliser (1/n)
V = numpy.cov(X,ddof=0,rowvar=False)
numpy.set_printoptions(precision=2,suppress=True)
print(V)
```

    [[132055.56   5732.54   5476.56   1181.78  38159.61   2850.87]
     [  5732.54    392.13    272.86     53.26   2017.05    197.27]
     [  5476.56    272.86    461.58     94.22   2482.36    120.64]
     [  1181.78     53.26     94.22     26.67    492.72     28.81]
     [ 38159.61   2017.05   2482.36    492.72  17715.36    749.99]
     [  2850.87    197.27    120.64     28.81    749.99    139.2 ]]
    


```python
#calculer la trace de la matrice V
trace = V.trace()
print(trace)
```

    150790.49382716054
    


```python
#vecteur moyenne pour (p = 6) variables
moyennes = numpy.mean(X,axis=0)

#inertie par l'écart au barycentre
print(numpy.mean(numpy.apply_along_axis(func1d=lambda x: numpy.sum((x-moyennes)**2),axis=1,arr=X)))
```

    150790.4938271605
    


```python
#matrice Z des variables centrées
Z = X - moyennes
print(pandas.DataFrame(Z,index=D.index))
```

                               0          1     2          3           4  \
    Modele                                                                 
    Alfasud TI       -281.666667  -5.611111 -40.5  -5.666667 -208.833333   
    Audi 100          -43.666667   0.388889  34.5  10.333333   31.166667   
    Simca 1300       -337.666667 -16.611111  -9.5   1.333333  -28.833333   
    Citroen GS Club  -409.666667 -25.611111 -21.5  -5.666667 -148.833333   
    Fiat 132          -46.666667  13.388889   5.5  -2.666667   26.166667   
    Lancia Beta      -334.666667  -2.611111  -4.5   2.333333    1.166667   
    Peugeot 504       164.333333  -5.611111  15.5   2.333333   81.166667   
    Renault 16 TL     -66.666667 -29.611111  -9.5  -3.666667  -68.833333   
    Renault 30       1032.333333  43.388889  18.5   6.333333  241.166667   
    Toyota Corolla   -465.666667 -29.611111 -34.5  -9.666667 -263.833333   
    Alfetta 1.66      -61.666667  24.388889  -5.5  -4.666667  -18.833333   
    Princess 1800     166.333333  -2.611111  11.5   5.333333   81.166667   
    Datsun 200L       366.333333  30.388889  35.5   2.333333  291.166667   
    Taunus 2000       361.333333  13.388889   4.5   3.333333    1.166667   
    Rancho           -189.666667  -4.611111  -2.5  -0.666667   50.166667   
    Mazda 9295        137.333333  -1.611111   6.5  -1.666667   16.166667   
    Opel Rekord       347.333333  15.388889  25.5   6.333333   41.166667   
    Lada 1300        -337.666667 -16.611111 -29.5  -5.666667 -123.833333   
    
                             5  
    Modele                      
    Alfasud TI        6.722222  
    Audi 100          1.722222  
    Simca 1300       -6.277778  
    Citroen GS Club  -7.277778  
    Fiat 132          6.722222  
    Lancia Beta       1.722222  
    Peugeot 504      -4.277778  
    Renault 16 TL   -18.277778  
    Renault 30       21.722222  
    Toyota Corolla  -18.277778  
    Alfetta 1.66     16.722222  
    Princess 1800    -0.277778  
    Datsun 200L       1.722222  
    Taunus 2000       8.722222  
    Rancho          -14.277778  
    Mazda 9295        6.722222  
    Opel Rekord      14.722222  
    Lada 1300       -18.277778  
    


```python
#(1/n) (Z'Z)
print(numpy.dot(numpy.transpose(Z),Z)/n)
```

    [[132055.56   5732.54   5476.56   1181.78  38159.61   2850.87]
     [  5732.54    392.13    272.86     53.26   2017.05    197.27]
     [  5476.56    272.86    461.58     94.22   2482.36    120.64]
     [  1181.78     53.26     94.22     26.67    492.72     28.81]
     [ 38159.61   2017.05   2482.36    492.72  17715.36    749.99]
     [  2850.87    197.27    120.64     28.81    749.99    139.2 ]]
    


```python
#matrice des corrélations
R = numpy.corrcoef(X,rowvar=False)
print(R)
```

    [[1.   0.8  0.7  0.63 0.79 0.66]
     [0.8  1.   0.64 0.52 0.77 0.84]
     [0.7  0.64 1.   0.85 0.87 0.48]
     [0.63 0.52 0.85 1.   0.72 0.47]
     [0.79 0.77 0.87 0.72 1.   0.48]
     [0.66 0.84 0.48 0.47 0.48 1.  ]]
    


```python
#vecteur ecart-type pour (p = 6) variables
sigmas = numpy.std(X,axis=0)

#centrage et réduction
Z = (X - moyennes)/sigmas

#correspondance : produit matriciel : (1/n) (Z'Z)
print(numpy.dot(numpy.transpose(Z),Z)/n)
```

    [[1.   0.8  0.7  0.63 0.79 0.66]
     [0.8  1.   0.64 0.52 0.77 0.84]
     [0.7  0.64 1.   0.85 0.87 0.48]
     [0.63 0.52 0.85 1.   0.72 0.47]
     [0.79 0.77 0.87 0.72 1.   0.48]
     [0.66 0.84 0.48 0.47 0.48 1.  ]]
    


```python
#heatmap pour identifier visuellement les corrélations fortes
sns.heatmap(R,xticklabels=D.columns,yticklabels=D.columns,vmin=-1,vmax=+1,center=0,cmap="RdBu",linewidths=0.5)
```




    <matplotlib.axes._subplots.AxesSubplot at 0x1e14f43a248>




![png](output_31_1.png)



```python
#acp sur Z (p = 6 variables)
acp = PCA()

#coordonnées factorielles
coord = acp.fit_transform(Z)

#afficher les nouvelles coordonnées des premiers véhicules
print(pandas.DataFrame(coord,index=Dbis.index).head())
```

                            0         1         2         3         4         5
    Modele                                                                     
    Toyota Corolla  -2.138924  1.785681  0.571862  0.201927 -0.301357  0.053921
    Citroen GS Club  1.561459 -1.527040  1.315271 -0.211352  0.148516 -0.327238
    Lada 1300       -1.119385 -0.674505  0.456588 -0.167626 -0.375364  0.271803
    Simca 1300      -2.573742  0.112884  0.148570 -0.017343  0.226853  0.263476
    Lancia Beta      0.427855  0.695567 -0.193286 -0.627754  0.263505 -0.037153
    


```python
#nombre de variables
p = Z.shape[1]
print(p)

#corrélation des facteurs avec les variables
Mlambda = numpy.corrcoef(x=coord,y=Z,rowvar=False)[:p,p:]

#affichage des corrélations : lignes = facteurs, colonnes = variables
print(Mlambda)
```

    6
    [[ 0.89  0.89  0.89  0.81  0.91  0.75]
     [ 0.11  0.38 -0.38 -0.41 -0.22  0.57]
     [-0.22 -0.11  0.04  0.37 -0.3   0.3 ]
     [ 0.37 -0.17 -0.13  0.1  -0.14 -0.03]
     [ 0.05 -0.09  0.22 -0.15 -0.09  0.06]
     [-0.01 -0.13 -0.04 -0.02  0.12  0.1 ]]
    


```python
#somme des corrélations^2 entre les variables et les facteurs
#c.-à-d. valeurs des lambda
slambda = numpy.sum(Mlambda**2,axis=1)
print(slambda)
```

    [4.42 0.86 0.37 0.21 0.09 0.04]
    


```python
#somme des lambda = inertie = p puisque ACP normée
print(numpy.sum(slambda))
```

    6.000000000000002
    


```python
#faire le lien avec le premier prisme
#variance des coordonnées factorielles
print(numpy.var(coord,axis=0,ddof=0))
```

    [4.42 0.86 0.37 0.21 0.09 0.04]
    


```python
#liste des variables
print(D.columns)
```

    Index(['CYL', 'PUISS', 'LONG', 'LARG', 'POIDS', 'VMAX'], dtype='object')
    


```python
#corrélation entre CYL et POIDS
print(R[0,4])
```

    0.7889520283865826
    


```python
#corrélation approchée sur le premier facteur
print(Mlambda[0,0]*Mlambda[0,4])
```

    0.8087519932967818
    


```python
#corrélation approchée sur les 2 premiers facteurs
print(Mlambda[0,0]*Mlambda[0,4]+Mlambda[1,0]*Mlambda[1,4])
```

    0.7829518411637988
    


```python
#corrélation entre POIDS et VMAX
print(R[4,5])
```

    0.4775955992662051
    


```python
#corrélation approchée sur le premier facteur
print(Mlambda[0,4]*Mlambda[0,5])

#corrélation approchée sur les 2 premiers facteurs
print(Mlambda[0,4]*Mlambda[0,5]+Mlambda[1,4]*Mlambda[1,5])
```

    0.6831543614480379
    0.55438062524932
    


```python
#composantes
print(acp.components_)

#
a = acp.components_[0]
print(a)

#produit
numpy.dot(numpy.dot(a,R),numpy.transpose(a))

#norme
print(numpy.linalg.norm(a))
```

    [[ 0.42  0.42  0.42  0.39  0.43  0.36]
     [ 0.12  0.42 -0.41 -0.45 -0.24  0.62]
     [-0.35 -0.18  0.07  0.6  -0.48  0.49]
     [ 0.81 -0.36 -0.28  0.21 -0.3  -0.07]
     [ 0.15 -0.29  0.73 -0.48 -0.3   0.19]
     [-0.06 -0.63 -0.19 -0.11  0.58  0.46]]
    [0.42 0.42 0.42 0.39 0.43 0.36]
    1.0000000000000002
    


```python
#rappel matrice des corrélations
print(R)

#calcul des valeurs et vecteurs propres 
sol = numpy.linalg.eig(R)
print(sol)
```

    [[1.   0.8  0.7  0.63 0.79 0.66]
     [0.8  1.   0.64 0.52 0.77 0.84]
     [0.7  0.64 1.   0.85 0.87 0.48]
     [0.63 0.52 0.85 1.   0.72 0.47]
     [0.79 0.77 0.87 0.72 1.   0.48]
     [0.66 0.84 0.48 0.47 0.48 1.  ]]
    (array([4.42, 0.86, 0.37, 0.21, 0.09, 0.04]), array([[-0.42, -0.12, -0.35,  0.81,  0.15, -0.06],
           [-0.42, -0.42, -0.18, -0.36, -0.29, -0.63],
           [-0.42,  0.41,  0.07, -0.28,  0.73, -0.19],
           [-0.39,  0.45,  0.6 ,  0.21, -0.48, -0.11],
           [-0.43,  0.24, -0.48, -0.3 , -0.3 ,  0.58],
           [-0.36, -0.62,  0.49, -0.07,  0.19,  0.46]]))
    


```python
#valeurs propres
print(sol[0])
```

    [4.42 0.86 0.37 0.21 0.09 0.04]
    


```python
#vecteurs propres pour le premier facteur
print(sol[1][:,0])
```

    [-0.42 -0.42 -0.42 -0.39 -0.43 -0.36]
    


```python
#vérifions sa norme
print(numpy.linalg.norm(sol[1][:,0]))
```

    0.9999999999999999
    


```python
#est-ce que la matrice est orthonormale ?
print(numpy.dot(numpy.transpose(sol[1]),sol[1]))
```

    [[ 1. -0.  0.  0. -0. -0.]
     [-0.  1.  0.  0.  0.  0.]
     [ 0.  0.  1. -0. -0. -0.]
     [ 0.  0. -0.  1. -0.  0.]
     [-0.  0. -0. -0.  1.  0.]
     [-0.  0. -0.  0.  0.  1.]]
    


```python
#coordonnées factorielles sur le premier facteur (n°0)
#produit matriciel
c1 = numpy.dot(Z,sol[1][:,0])

#affichage avec les identifiants des véhicules
print(pandas.DataFrame(c1,columns=['F_1'],index=D.index))
```

                          F_1
    Modele                   
    Alfasud TI       2.138924
    Audi 100        -1.561459
    Simca 1300       1.119385
    Citroen GS Club  2.573742
    Fiat 132        -0.427855
    Lancia Beta      0.304238
    Peugeot 504     -0.683928
    Renault 16 TL    1.948493
    Renault 30      -4.409735
    Toyota Corolla   3.985782
    Alfetta 1.66    -0.437658
    Princess 1800   -1.018175
    Datsun 200L     -2.941080
    Taunus 2000     -1.314880
    Rancho           0.691111
    Mazda 9295      -0.385709
    Opel Rekord     -2.289768
    Lada 1300        2.708574
    


```python
#avec l'objet PCA de scikit-learn
print((17/18)*acp.explained_variance_)
```

    [4.42 0.86 0.37 0.21 0.09 0.04]
    


```python
#corrélation des variables avec le premier facteur
r_1 = numpy.sqrt(sol[0][0])*sol[1][:,0]

#affichage
print(pandas.DataFrame(r_1,index=D.columns))
```

                  0
    CYL   -0.893464
    PUISS -0.886858
    LONG  -0.886155
    LARG  -0.813536
    POIDS -0.905187
    VMAX  -0.754710
    


```python
#décomposition en valeurs singulières
Us, delta, Vs = numpy.linalg.svd(Z)
```


```python
#vecteur des valeurs singulières
print(delta)

#et pour obtenir les lambda_k
print((delta**2)/n)
```

    [8.92 3.93 2.59 1.96 1.29 0.88]
    [4.42 0.86 0.37 0.21 0.09 0.04]
    


```python
#matrice singulière à droite (d'entrée)
print(numpy.transpose(Vs))

#vérification -- vecteurs propres de (Z'Z) -- OK 
print(numpy.linalg.eig(numpy.dot(numpy.transpose(Z),Z))[1])
```

    [[ 0.42 -0.12  0.35 -0.81 -0.15  0.06]
     [ 0.42 -0.42  0.18  0.36  0.29  0.63]
     [ 0.42  0.41 -0.07  0.28 -0.73  0.19]
     [ 0.39  0.45 -0.6  -0.21  0.48  0.11]
     [ 0.43  0.24  0.48  0.3   0.3  -0.58]
     [ 0.36 -0.62 -0.49  0.07 -0.19 -0.46]]
    [[-0.42 -0.12 -0.35  0.81  0.15 -0.06]
     [-0.42 -0.42 -0.18 -0.36 -0.29 -0.63]
     [-0.42  0.41  0.07 -0.28  0.73 -0.19]
     [-0.39  0.45  0.6   0.21 -0.48 -0.11]
     [-0.43  0.24 -0.48 -0.3  -0.3   0.58]
     [-0.36 -0.62  0.49 -0.07  0.19  0.46]]
    


```python
#matrice singulière à gauche (de sortie)
print(Us)
```

    [[-0.24 -0.45 -0.22 -0.1   0.23 -0.06  0.31  0.21  0.17 -0.23 -0.2   0.14
       0.4  -0.03 -0.04  0.24  0.17 -0.3 ]
     [ 0.18  0.39 -0.51  0.11 -0.11  0.37  0.09  0.25  0.42 -0.06 -0.11  0.1
      -0.04 -0.06 -0.02  0.19 -0.29  0.05]
     [-0.13  0.17 -0.18  0.09  0.29 -0.31 -0.16 -0.16  0.43  0.12  0.47 -0.25
       0.    0.24 -0.29  0.16  0.17 -0.09]
     [-0.29 -0.03 -0.06  0.01 -0.18 -0.3  -0.09 -0.17  0.45 -0.2  -0.31  0.21
      -0.06  0.26  0.22 -0.45 -0.02  0.24]
     [ 0.05 -0.18  0.07  0.32 -0.2   0.04  0.08  0.19  0.04  0.15 -0.27  0.11
      -0.55  0.27 -0.34  0.12  0.36 -0.15]
     [-0.03  0.05 -0.26  0.28  0.34 -0.23  0.09  0.33 -0.23  0.51 -0.23 -0.13
       0.14  0.1   0.17 -0.15  0.03  0.31]
     [ 0.08  0.24  0.1  -0.1  -0.16 -0.17  0.89 -0.13  0.03  0.05  0.13 -0.08
      -0.02  0.07 -0.01 -0.09  0.04  0.05]
     [-0.22  0.25  0.24 -0.32 -0.23 -0.12 -0.11  0.76  0.03 -0.08  0.18 -0.06
       0.05  0.07 -0.01 -0.09  0.1  -0.04]
     [ 0.49 -0.27  0.23 -0.43  0.29 -0.05  0.    0.12  0.44  0.25 -0.1  -0.04
      -0.19 -0.14  0.06 -0.04 -0.05  0.11]
     [-0.45 -0.06  0.12 -0.14 -0.22  0.37  0.02 -0.15  0.21  0.68  0.03  0.1
       0.15 -0.04  0.04 -0.   -0.02 -0.12]
     [ 0.05 -0.49 -0.01  0.39 -0.13  0.06  0.1   0.22  0.1  -0.02  0.61  0.18
      -0.05 -0.02  0.18 -0.12 -0.18  0.18]
     [ 0.11  0.21 -0.08 -0.15  0.14 -0.21 -0.05 -0.03 -0.15  0.14  0.16  0.87
      -0.    0.01 -0.07  0.05  0.09 -0.02]
     [ 0.33  0.14  0.48  0.39 -0.04  0.06 -0.05  0.01  0.21 -0.01 -0.13  0.1
       0.6   0.16 -0.15 -0.02  0.06  0.  ]
     [ 0.15 -0.12 -0.11 -0.3   0.05  0.29 -0.01 -0.02 -0.18 -0.02  0.07 -0.02
       0.08  0.84  0.08  0.02 -0.13  0.02]
     [-0.08  0.23  0.24  0.18  0.29  0.14  0.04  0.01  0.1  -0.08  0.03  0.04
      -0.21  0.08  0.72  0.21  0.2  -0.26]
     [ 0.04 -0.09  0.03 -0.05 -0.41 -0.38 -0.09 -0.05  0.    0.11 -0.07 -0.03
       0.07  0.06  0.22  0.71 -0.09  0.27]
     [ 0.26 -0.03 -0.31 -0.12 -0.26  0.18 -0.09 -0.08  0.02  0.    0.12 -0.07
       0.17 -0.1   0.19 -0.11  0.76  0.18]
     [-0.3   0.04  0.22 -0.05  0.3   0.32  0.09 -0.01  0.02 -0.19  0.01  0.08
      -0.07 -0.02 -0.21  0.22  0.16  0.7 ]]
    


```python
#coordonnées factorielles - 1er facteur
print(pandas.DataFrame(Us[:,0]*svd[1][0],columns=['F_1'],index=D.index))
```

                          F_1
    Modele                   
    Alfasud TI      -2.138924
    Audi 100         1.561459
    Simca 1300      -1.119385
    Citroen GS Club -2.573742
    Fiat 132         0.427855
    Lancia Beta     -0.304238
    Peugeot 504      0.683928
    Renault 16 TL   -1.948493
    Renault 30       4.409735
    Toyota Corolla  -3.985782
    Alfetta 1.66     0.437658
    Princess 1800    1.018175
    Datsun 200L      2.941080
    Taunus 2000      1.314880
    Rancho          -0.691111
    Mazda 9295       0.385709
    Opel Rekord      2.289768
    Lada 1300       -2.708574
    


```python

```
