```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement des données - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="DATA_ACP_ACTIF",index_col=0)

#affichage des caractéristiques
print(D.info())

#nombre d'observations
n = D.shape[0]

#matrice des X sans la variable CYL
X = D.iloc[:,1:].values
print(X)

#nombre de variables -- p = 5 maintenant
p = X.shape[1]
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 18 entries, Alfasud TI to Lada 1300
    Data columns (total 6 columns):
     #   Column  Non-Null Count  Dtype
    ---  ------  --------------  -----
     0   CYL     18 non-null     int64
     1   PUISS   18 non-null     int64
     2   LONG    18 non-null     int64
     3   LARG    18 non-null     int64
     4   POIDS   18 non-null     int64
     5   VMAX    18 non-null     int64
    dtypes: int64(6)
    memory usage: 1008.0+ bytes
    None
    [[  79  393  161  870  165]
     [  85  468  177 1110  160]
     [  68  424  168 1050  152]
     [  59  412  161  930  151]
     [  98  439  164 1105  165]
     [  82  429  169 1080  160]
     [  79  449  169 1160  154]
     [  55  424  163 1010  140]
     [ 128  452  173 1320  180]
     [  55  399  157  815  140]
     [ 109  428  162 1060  175]
     [  82  445  172 1160  158]
     [ 115  469  169 1370  160]
     [  98  438  170 1080  167]
     [  80  431  166 1129  144]
     [  83  440  165 1095  165]
     [ 100  459  173 1120  173]
     [  68  404  161  955  140]]
    


```python
#librairie numpy
import numpy

#matrice des corrélations
R = numpy.corrcoef(X,rowvar=False)
print(R)
```

    [[1.         0.64136235 0.52083197 0.765293   0.84437948]
     [0.64136235 1.         0.84926635 0.86809028 0.47592847]
     [0.52083197 0.84926635 1.         0.71687392 0.47294527]
     [0.765293   0.86809028 0.71687392 1.         0.4775956 ]
     [0.84437948 0.47592847 0.47294527 0.4775956  1.        ]]
    


```python
#librairie graphique
import seaborn as sns

#heatmap pour identifier visuellement les corrélations fortes
sns.heatmap(R,xticklabels=D.columns[1:],yticklabels=D.columns[1:],vmin=-1,vmax=+1,center=0,cmap="RdBu",linewidths=0.5)
```




    <matplotlib.axes._subplots.AxesSubplot at 0x191d480c688>




![png](output_3_1.png)



```python
#variable de contrôle
y = D.CYL.values

#librairie numpy
import numpy

#calculer le résidu de la régression de chaque variable avec CYL
E = numpy.apply_along_axis(arr=X,axis=0,func1d=lambda x:x-numpy.polyval(numpy.polyfit(y,x,1),y))
print(E.shape)

#matrice des corrélations partielles vs. CYL
RPART = numpy.corrcoef(E,rowvar=False)
print(RPART)
```

    (18, 5)
    [[ 1.          0.19163511  0.04078385  0.36829487  0.69698446]
     [ 0.19163511  1.          0.736086    0.71854683  0.01785118]
     [ 0.04078385  0.736086    1.          0.46097645  0.09341514]
     [ 0.36829487  0.71854683  0.46097645  1.         -0.1024223 ]
     [ 0.69698446  0.01785118  0.09341514 -0.1024223   1.        ]]
    


```python
#heatmap pour identifier visuellement les corrélations partielles fortes
sns.heatmap(RPART,xticklabels=D.columns[1:],yticklabels=D.columns[1:],vmin=-1,vmax=+1,center=0,cmap="RdBu",linewidths=0.5)
```




    <matplotlib.axes._subplots.AxesSubplot at 0x191d4889388>




![png](output_5_1.png)



```python
#ACP sur les résidus de la régression sur CYL
from fanalysis.pca import PCA
acp = PCA(std_unit=True,row_labels=D.index,col_labels=D.columns[1:])
acp.fit(E)
```




    PCA(col_labels=Index(['PUISS', 'LONG', 'LARG', 'POIDS', 'VMAX'], dtype='object'),
        n_components=None,
        row_labels=Index(['Alfasud TI', 'Audi 100', 'Simca 1300', 'Citroen GS Club', 'Fiat 132',
           'Lancia Beta', 'Peugeot 504', 'Renault 16 TL', 'Renault 30',
           'Toyota Corolla', 'Alfetta 1.66', 'Princess 1800', 'Datsun 200L',
           'Taunus 2000', 'Rancho', 'Mazda 9295', 'Opel Rekord', 'Lada 1300'],
          dtype='object', name='Modele'),
        stats=True, std_unit=True)




```python
#les valeurs propres
print(acp.eig_[0])
```

    [2.40105829 1.61317102 0.69868647 0.18349256 0.10359165]
    


```python
#librairie graphique
import matplotlib.pyplot as plt

#éboulis des v.p.
fig,ax=plt.subplots(figsize=(5,5))
ax.plot(range(1,p+1),acp.eig_[0],".-")
ax.set_xlabel("Nb. facteurs")
ax.set_ylabel("Val. propres")
plt.title("Eboulis des v.p.")
#seuil de Kaiser
ax.plot([1,p],[1,1],"r--",linewidth=1)
plt.show()
```


![png](output_8_0.png)



```python
#cercle des corrélations
acp.correlation_circle(num_x_axis=1,num_y_axis=2,figsize=(5,5))
```


![png](output_9_0.png)



```python
#coordonnées des individus
print(acp.row_coord_[:,:2])
```

    [[ 1.92106244  2.15849617]
     [-3.11652858 -0.81588609]
     [-1.13242886 -0.43878504]
     [ 0.79774951 -0.06420984]
     [-0.80256939  1.37314182]
     [-2.1350403   0.75180472]
     [-0.18503267 -1.51895334]
     [ 1.96469676 -2.40158866]
     [ 1.70939618  0.47096097]
     [ 2.50386059 -0.41669294]
     [-0.39806347  3.12261315]
     [-0.54585244 -1.1523552 ]
     [-2.1345301  -0.2900653 ]
     [ 1.1051276   0.27571894]
     [-0.93203315 -0.93624777]
     [ 0.61625559  0.08588495]
     [-0.5758171   0.37301803]
     [ 1.33974736 -0.57685457]]
    


```python
#représentation des individus
fig,ax = plt.subplots(figsize=(10,10))
ax.axis([-3.5,+3.5,-3.5,+3.5])
ax.plot([-3.5,+3.5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-3.5,+3.5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des individus")

for i in range(n):
    ax.text(acp.row_coord_[i,0],acp.row_coord_[i,1],D.index[i],color='navy')

plt.show()
```


![png](output_11_0.png)

