```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement des données - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="DATA_ACP_ACTIF",index_col=0)

#affichage des caractéristiques
print(D.info())

#nombre de variables
p = D.shape[1]

#nombre d'observations
n = D.shape[0]

#matrice des X
X = D.values
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 18 entries, Alfasud TI to Lada 1300
    Data columns (total 6 columns):
     #   Column  Non-Null Count  Dtype
    ---  ------  --------------  -----
     0   CYL     18 non-null     int64
     1   PUISS   18 non-null     int64
     2   LONG    18 non-null     int64
     3   LARG    18 non-null     int64
     4   POIDS   18 non-null     int64
     5   VMAX    18 non-null     int64
    dtypes: int64(6)
    memory usage: 1008.0+ bytes
    None
    


```python
#importer la classe de calcul
from fanalysis.pca import PCA

#instancier l'objet de calcul
acp = PCA(std_unit=True,row_labels=D.index,col_labels=D.columns)

#lancer les calculs
acp.fit(X)

#propriétés de l'objet
print(dir(acp))
```

    ['__class__', '__delattr__', '__dict__', '__dir__', '__doc__', '__eq__', '__format__', '__ge__', '__getattribute__', '__getstate__', '__gt__', '__hash__', '__init__', '__init_subclass__', '__le__', '__lt__', '__module__', '__ne__', '__new__', '__reduce__', '__reduce_ex__', '__repr__', '__setattr__', '__setstate__', '__sizeof__', '__str__', '__subclasshook__', '__weakref__', '_compute_stats', '_compute_svd', '_get_param_names', '_get_tags', '_more_tags', 'col_contrib_', 'col_coord_', 'col_cor_', 'col_cos2_', 'col_labels', 'col_labels_', 'col_labels_short_', 'col_topandas', 'correlation_circle', 'eig_', 'eigen_vectors_', 'fit', 'fit_transform', 'get_params', 'mapping', 'mapping_col', 'mapping_row', 'means_', 'model_', 'n_components', 'n_components_', 'plot_col_contrib', 'plot_col_cos2', 'plot_eigenvalues', 'plot_row_contrib', 'plot_row_cos2', 'row_contrib_', 'row_coord_', 'row_cos2_', 'row_labels', 'row_labels_', 'row_topandas', 'set_params', 'ss_col_coord_', 'stats', 'std_', 'std_unit', 'transform']
    


```python
#valeurs propres
print(acp.eig_)
```

    [[4.42085806e+00 8.56062289e-01 3.73066077e-01 2.13922089e-01
      9.28012120e-02 4.32902727e-02]
     [7.36809677e+01 1.42677048e+01 6.21776796e+00 3.56536815e+00
      1.54668687e+00 7.21504545e-01]
     [7.36809677e+01 8.79486725e+01 9.41664404e+01 9.77318086e+01
      9.92784955e+01 1.00000000e+02]]
    


```python
%matplotlib inline

#librairie graphique
import matplotlib.pyplot as plt

#préparer le graphique
fig, ax = plt.subplots(figsize=(5,5))
ax.plot(range(1,p+1),acp.eig_[0],".-")
ax.set_xlabel("Nb. facteurs")
ax.set_ylabel("Val. propres")
plt.title("Eboulis des valeurs propres")

#rajout du seuil du Kaiser
ax.plot([1,6],[1,1],"r--",linewidth=1)

plt.show()
```


![png](output_4_0.png)



```python
#librairie pour calculs matriciels
import numpy

#proportion de variance expliquée
fig, ax = plt.subplots(figsize=(5,5))
ax.plot(range(0,p+1),numpy.append(0,acp.eig_[2]),".-")
ax.set_xlabel("Nb. facteurs")
ax.set_ylabel("% de var. expliquée")
plt.title("Variance expliquée")

plt.show()
```


![png](output_5_0.png)



```python
#librairie pour calculs statistiques
import scipy.stats as stats

#stat. de test Bartlett
C = -(n-1-(2*p+5)/6) * numpy.sum(numpy.log(acp.eig_[0]))
print(C)

#degrés de liberté
ddl = p*(p-1)/2

#p-value
print(1-stats.chi2.cdf(C,df=ddl))
```

    95.11987830353813
    1.0891287871572786e-13
    


```python
#librairie pour calc. math
import math

#seuil de Karlis-Saporta-Spinaki
kss = 1+2*math.sqrt((p-1)/(n-1))
print(kss)
```

    2.084652289093281
    


```python
#seuils pour test des bâtons brisés
b = numpy.flip(numpy.cumsum(1/numpy.arange(p,0,-1)))
print(b)

#vérification des v.p. qui passent
fig, ax = plt.subplots(figsize=(5,5))
ax.plot(range(1,p+1),acp.eig_[0],".-")
ax.set_xlabel("Nb. facteurs")
ax.set_ylabel("Val. propres")
plt.title("Val. propres et seuils des bâtons brisés")

#rajout du seuil des bâtons brisés
ax.plot(range(1,p+1),b,"r--",linewidth=1)

plt.show()
```

    [2.45       1.45       0.95       0.61666667 0.36666667 0.16666667]
    


![png](output_8_1.png)



```python
#corrélation des variables avec les facteurs
print(pandas.DataFrame(acp.col_coord_[:,:2],index=D.columns))
```

                  0         1
    CYL    0.893464 -0.114906
    PUISS  0.886858 -0.384689
    LONG   0.886155  0.381029
    LARG   0.813536  0.412736
    POIDS  0.905187  0.224532
    VMAX   0.754710 -0.573519
    


```python
#cercle des corrélations
acp.correlation_circle(num_x_axis=1,num_y_axis=2)
```


![png](output_10_0.png)



```python
#cos2 des variables sur les 2 facteurs
print(pandas.DataFrame(acp.col_cos2_[:,:2],index=D.columns))
```

                  0         1
    CYL    0.798277  0.013203
    PUISS  0.786517  0.147986
    LONG   0.785270  0.145183
    LARG   0.661841  0.170351
    POIDS  0.819364  0.050415
    VMAX   0.569588  0.328925
    


```python
#cos2 cumulé sur les 2 premiers facteurs
print(pandas.DataFrame(numpy.cumsum(acp.col_cos2_[:,:2],axis=1),index=D.columns))
```

                  0         1
    CYL    0.798277  0.811481
    PUISS  0.786517  0.934503
    LONG   0.785270  0.930453
    LARG   0.661841  0.832192
    POIDS  0.819364  0.869779
    VMAX   0.569588  0.898512
    


```python
#contributions sur les 2 premiers facteurs (en %)
print(pandas.DataFrame(acp.col_contrib_[:,:2],index=D.columns))
```

                   0          1
    CYL    18.057062   1.542342
    PUISS  17.791052  17.286793
    LONG   17.762847  16.959384
    LARG   14.970882  19.899361
    POIDS  18.534057   5.889155
    VMAX   12.884099  38.422964
    


```python
#coordonnées factorielles des individus dans le plan
print(pandas.DataFrame(acp.row_coord_[:,:2],index=D.index))
```

                            0         1
    Modele                             
    Alfasud TI      -2.138924 -1.785681
    Audi 100         1.561459  1.527040
    Simca 1300      -1.119385  0.674505
    Citroen GS Club -2.573742 -0.112884
    Fiat 132         0.427855 -0.695567
    Lancia Beta     -0.304238  0.196149
    Peugeot 504      0.683928  0.933057
    Renault 16 TL   -1.948493  0.980448
    Renault 30       4.409735 -1.063633
    Toyota Corolla  -3.985782 -0.236240
    Alfetta 1.66     0.437658 -1.912448
    Princess 1800    1.018175  0.841712
    Datsun 200L      2.941080  0.559175
    Taunus 2000      1.314880 -0.486522
    Rancho          -0.691111  0.897721
    Mazda 9295       0.385709 -0.356185
    Opel Rekord      2.289768 -0.104345
    Lada 1300       -2.708574  0.143699
    


```python
#carte des individus
acp.mapping_row(num_x_axis=1,num_y_axis=2,figsize=(10,10))
```


![png](output_15_0.png)



```python
#graphique avec les échelles carrées
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(acp.row_coord_[:,0],acp.row_coord_[:,1],'wo')
ax.axis([-5,+5,-5,+5])
ax.plot([-5,+5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-5,+5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1 (73.68%)")
ax.set_ylabel("Dim.2 (14.27%)")
plt.title("Carte des individus")

for i in range(n):
    ax.text(acp.row_coord_[i,0],acp.row_coord_[i,1],D.index[i])
    
plt.show()
```


![png](output_16_0.png)



```python
#données cenrrées et réduites
Z = (D.values - acp.means_) / acp.std_

#distance à l'origine des individus dans le repère initial
disto = numpy.apply_along_axis(arr=Z,axis=1,func1d=lambda x: numpy.sum(x**2))

#distance à l'origine à partir des coordonnées factorielles
distoBis = numpy.sum(acp.row_coord_**2,axis=1)

#affichage de contrôle
print(pandas.DataFrame(numpy.transpose(numpy.array([disto,distoBis])),index=D.index))
```

                             0          1
    Modele                               
    Alfasud TI        8.225176   8.225176
    Audi 100          6.673755   6.673755
    Simca 1300        2.159327   2.159327
    Citroen GS Club   6.780145   6.780145
    Fiat 132          1.169124   1.169124
    Lancia Beta       1.134950   1.134950
    Peugeot 504       1.512793   1.512793
    Renault 16 TL     5.636826   5.636826
    Renault 30       21.789657  21.789657
    Toyota Corolla   16.290143  16.290143
    Alfetta 1.66      4.456770   4.456770
    Princess 1800     1.952513   1.952513
    Datsun 200L      11.112624  11.112624
    Taunus 2000       2.452986   2.452986
    Rancho            1.963373   1.963373
    Mazda 9295        0.684521   0.684521
    Opel Rekord       6.083119   6.083119
    Lada 1300         7.922198   7.922198
    


```python
#cos2 des individus
lig_cos2 = numpy.apply_along_axis(arr=acp.row_coord_[:,:2],axis=0,func1d=lambda x: (x**2)/disto)
print(pandas.DataFrame(lig_cos2,index=D.index))
```

                            0         1
    Modele                             
    Alfasud TI       0.556218  0.387670
    Audi 100         0.365334  0.349406
    Simca 1300       0.580284  0.210694
    Citroen GS Club  0.976992  0.001879
    Fiat 132         0.156579  0.413826
    Lancia Beta      0.081555  0.033900
    Peugeot 504      0.309202  0.575488
    Renault 16 TL    0.673539  0.170535
    Renault 30       0.892431  0.051920
    Toyota Corolla   0.975219  0.003426
    Alfetta 1.66     0.042978  0.820652
    Princess 1800    0.530947  0.362855
    Datsun 200L      0.778390  0.028137
    Taunus 2000      0.704819  0.096496
    Rancho           0.243273  0.410469
    Mazda 9295       0.217336  0.185337
    Opel Rekord      0.861900  0.001790
    Lada 1300        0.926052  0.002607
    


```python
#affichage de l'outil PCA
print(pandas.DataFrame(acp.row_cos2_[:,:2],index=D.index))
```

                            0         1
    Modele                             
    Alfasud TI       0.556218  0.387670
    Audi 100         0.365334  0.349406
    Simca 1300       0.580284  0.210694
    Citroen GS Club  0.976992  0.001879
    Fiat 132         0.156579  0.413826
    Lancia Beta      0.081555  0.033900
    Peugeot 504      0.309202  0.575488
    Renault 16 TL    0.673539  0.170535
    Renault 30       0.892431  0.051920
    Toyota Corolla   0.975219  0.003426
    Alfetta 1.66     0.042978  0.820652
    Princess 1800    0.530947  0.362855
    Datsun 200L      0.778390  0.028137
    Taunus 2000      0.704819  0.096496
    Rancho           0.243273  0.410469
    Mazda 9295       0.217336  0.185337
    Opel Rekord      0.861900  0.001790
    Lada 1300        0.926052  0.002607
    


```python
#cumul sur les 2 premiers facteurs
print(pandas.DataFrame(numpy.cumsum(acp.row_cos2_[:,:2],axis=1),index=D.index))
```

                            0         1
    Modele                             
    Alfasud TI       0.556218  0.943889
    Audi 100         0.365334  0.714741
    Simca 1300       0.580284  0.790978
    Citroen GS Club  0.976992  0.978871
    Fiat 132         0.156579  0.570405
    Lancia Beta      0.081555  0.115454
    Peugeot 504      0.309202  0.884690
    Renault 16 TL    0.673539  0.844075
    Renault 30       0.892431  0.944351
    Toyota Corolla   0.975219  0.978645
    Alfetta 1.66     0.042978  0.863630
    Princess 1800    0.530947  0.893802
    Datsun 200L      0.778390  0.806527
    Taunus 2000      0.704819  0.801315
    Rancho           0.243273  0.653742
    Mazda 9295       0.217336  0.402674
    Opel Rekord      0.861900  0.863690
    Lada 1300        0.926052  0.928659
    


```python
#contributions sur les 2 premiers facteurs
lig_ctr = (acp.row_coord_[:,:2]**2)/(n*acp.eig_[0][:2])*100
print(pandas.DataFrame(lig_ctr,index=D.index))
```

                             0          1
    Modele                               
    Alfasud TI        5.749254  20.693307
    Audi 100          3.063951  15.132933
    Simca 1300        1.574636   2.952519
    Citroen GS Club   8.324360   0.082697
    Fiat 132          0.230046   3.139789
    Lancia Beta       0.116318   0.249686
    Peugeot 504       0.587817   5.649867
    Renault 16 TL     4.771099   6.238372
    Renault 30       24.436884   7.341856
    Toyota Corolla   19.964025   0.362185
    Alfetta 1.66      0.240708  23.735669
    Princess 1800     1.302765   4.597791
    Datsun 200L      10.870129   2.029163
    Taunus 2000       2.172668   1.536130
    Rancho            0.600229   5.230043
    Mazda 9295        0.186956   0.823327
    Opel Rekord       6.588764   0.070658
    Lada 1300         9.219391   0.134007
    


```python
#vérification avec l'outil PCA
print(pandas.DataFrame(acp.row_contrib_[:,:2],index=D.index))
```

                             0          1
    Modele                               
    Alfasud TI        5.749254  20.693307
    Audi 100          3.063951  15.132933
    Simca 1300        1.574636   2.952519
    Citroen GS Club   8.324360   0.082697
    Fiat 132          0.230046   3.139789
    Lancia Beta       0.116318   0.249686
    Peugeot 504       0.587817   5.649867
    Renault 16 TL     4.771099   6.238372
    Renault 30       24.436884   7.341856
    Toyota Corolla   19.964025   0.362185
    Alfetta 1.66      0.240708  23.735669
    Princess 1800     1.302765   4.597791
    Datsun 200L      10.870129   2.029163
    Taunus 2000       2.172668   1.536130
    Rancho            0.600229   5.230043
    Mazda 9295        0.186956   0.823327
    Opel Rekord       6.588764   0.070658
    Lada 1300         9.219391   0.134007
    


```python
#affichage graphique - 1er facteur
acp.plot_row_contrib(num_axis=1,nb_values=5)
```


![png](output_23_0.png)



```python
#affichage graphique - 1e facteur
acp.plot_row_contrib(num_axis=2,nb_values=5)
```


![png](output_24_0.png)



```python
#coordonnées des individus à partir des variables - Facteur 1
f1 = 1/math.sqrt(acp.eig_[0][0]) * numpy.dot(Z,acp.col_coord_[:,0])

#affichage + vérification avec coordonnées de PCA
print(pandas.DataFrame(numpy.transpose(numpy.array([f1,acp.row_coord_[:,0]])),index=D.index))
```

                            0         1
    Modele                             
    Alfasud TI      -2.138924 -2.138924
    Audi 100         1.561459  1.561459
    Simca 1300      -1.119385 -1.119385
    Citroen GS Club -2.573742 -2.573742
    Fiat 132         0.427855  0.427855
    Lancia Beta     -0.304238 -0.304238
    Peugeot 504      0.683928  0.683928
    Renault 16 TL   -1.948493 -1.948493
    Renault 30       4.409735  4.409735
    Toyota Corolla  -3.985782 -3.985782
    Alfetta 1.66     0.437658  0.437658
    Princess 1800    1.018175  1.018175
    Datsun 200L      2.941080  2.941080
    Taunus 2000      1.314880  1.314880
    Rancho          -0.691111 -0.691111
    Mazda 9295       0.385709  0.385709
    Opel Rekord      2.289768  2.289768
    Lada 1300       -2.708574 -2.708574
    


```python
#coordonnées des variables à partir des individus
r1 = 1/(n*math.sqrt(acp.eig_[0][0])) * numpy.dot(numpy.transpose(Z),acp.row_coord_[:,0])

#affichage + vérification avec corrélations de PCA
print(pandas.DataFrame(numpy.transpose(numpy.array([r1,acp.col_coord_[:,0]])),index=D.columns))
```

                  0         1
    CYL    0.893464  0.893464
    PUISS  0.886858  0.886858
    LONG   0.886155  0.886155
    LARG   0.813536  0.813536
    POIDS  0.905187  0.905187
    VMAX   0.754710  0.754710
    


```python
#vecteurs propres (q = 2) premiers facteurs
print(acp.eigen_vectors_[:,:2])
```

    [[ 0.42493602 -0.12419108]
     [ 0.42179441 -0.41577389]
     [ 0.42145993  0.41181773]
     [ 0.38692224  0.446087  ]
     [ 0.43051198  0.24267581]
     [ 0.35894427 -0.6198626 ]]
    


```python
#valeurs centrées et réduites reconstituées
ZC = numpy.dot(acp.row_coord_[:,:2],numpy.transpose(acp.eigen_vectors_[:,:2]))
print(ZC)
```

    [[-0.68714004 -0.15974647 -1.6368457  -1.6241662  -1.35417384  0.3391225 ]
     [ 0.47387522  0.02371103  1.28695448  1.28535587  1.04280237 -0.38607854]
     [-0.55943466 -0.75259209 -0.19400286 -0.13222707 -0.31822269 -0.81989747]
     [-1.07965636 -1.03865558 -1.1312167  -1.04619406 -1.1354209  -0.85385718]
     [ 0.26819434  0.46966566 -0.10612313 -0.14473684  0.01539943  0.58473234]
     [-0.15364146 -0.20987928 -0.04744643 -0.03021689 -0.08337739 -0.23078966]
     [ 0.17474853 -0.09946343  0.67249777  0.68085164  0.52086972 -0.33287478]
     [-0.94974758 -1.22950797 -0.41744566 -0.31654998 -0.60091837 -1.30714333]
     [ 2.00594882  2.30223229  1.42050343  1.23175145  1.64032553  2.2421554 ]
     [-1.66436358 -1.58295814 -1.77713557 -1.64757164 -1.77325693 -1.28423718]
     [ 0.42348573  0.97974779 -0.60312467 -0.68377858 -0.27568782  1.34254999]
     [ 0.32812626  0.07949877  0.77575209  0.76943152  0.64259987 -0.15627763]
     [ 1.18032648  1.00804104  1.46982552  1.3874099   1.40186847  0.70907252]
     [ 0.61916179  0.75689253  0.35381082  0.29172512  0.44800453  0.77354586]
     [-0.40516711 -0.66475603  0.07842183  0.13305548 -0.07967646 -0.80453439]
     [ 0.20813654  0.31078209  0.01587771 -0.00964997  0.07961491  0.35923348]
     [ 0.98596369  1.00919522  0.92207459  0.83941547  0.96045074  0.88657854]
     [-1.16881662 -1.20220742 -1.08237754 -0.98390521 -1.13120118 -1.06130051]]
    


```python
#moyennes
print(acp.means_)

#écarts-type
print(acp.std_)

#déstandardisation et décentrage
XC = ZC.copy()

#boucle sur les variables
for j in range(p):
    XC[:,j] = ZC[:,j] * acp.std_[0][j] + acp.means_[0][j]
    
#print des valeurs approchées
print(pandas.DataFrame(XC,columns=D.columns,index=D.index).round(1))

#print des valeurs initiales
print(D)
```

    [[1631.66666667   84.61111111  433.5         166.66666667 1078.83333333
       158.27777778]]
    [[363.39449027  19.80218531  21.48449053   5.16397779 133.09906503
       11.79833112]]
                        CYL  PUISS   LONG   LARG   POIDS   VMAX
    Modele                                                     
    Alfasud TI       1382.0   81.4  398.3  158.3   898.6  162.3
    Audi 100         1803.9   85.1  461.1  173.3  1217.6  153.7
    Simca 1300       1428.4   69.7  429.3  166.0  1036.5  148.6
    Citroen GS Club  1239.3   64.0  409.2  161.3   927.7  148.2
    Fiat 132         1729.1   93.9  431.2  165.9  1080.9  165.2
    Lancia Beta      1575.8   80.5  432.5  166.5  1067.7  155.6
    Peugeot 504      1695.2   82.6  447.9  170.2  1148.2  154.4
    Renault 16 TL    1286.5   60.3  424.5  165.0   998.9  142.9
    Renault 30       2360.6  130.2  464.0  173.0  1297.2  184.7
    Toyota Corolla   1026.8   53.3  395.3  158.2   842.8  143.1
    Alfetta 1.66     1785.6  104.0  420.5  163.1  1042.1  174.1
    Princess 1800    1750.9   86.2  450.2  170.6  1164.4  156.4
    Datsun 200L      2060.6  104.6  465.1  173.8  1265.4  166.6
    Taunus 2000      1856.7   99.6  441.1  168.2  1138.5  167.4
    Rancho           1484.4   71.4  435.2  167.4  1068.2  148.8
    Mazda 9295       1707.3   90.8  433.8  166.6  1089.4  162.5
    Opel Rekord      1990.0  104.6  453.3  171.0  1206.7  168.7
    Lada 1300        1206.9   60.8  410.2  161.6   928.3  145.8
                      CYL  PUISS  LONG  LARG  POIDS  VMAX
    Modele                                               
    Alfasud TI       1350     79   393   161    870   165
    Audi 100         1588     85   468   177   1110   160
    Simca 1300       1294     68   424   168   1050   152
    Citroen GS Club  1222     59   412   161    930   151
    Fiat 132         1585     98   439   164   1105   165
    Lancia Beta      1297     82   429   169   1080   160
    Peugeot 504      1796     79   449   169   1160   154
    Renault 16 TL    1565     55   424   163   1010   140
    Renault 30       2664    128   452   173   1320   180
    Toyota Corolla   1166     55   399   157    815   140
    Alfetta 1.66     1570    109   428   162   1060   175
    Princess 1800    1798     82   445   172   1160   158
    Datsun 200L      1998    115   469   169   1370   160
    Taunus 2000      1993     98   438   170   1080   167
    Rancho           1442     80   431   166   1129   144
    Mazda 9295       1769     83   440   165   1095   165
    Opel Rekord      1979    100   459   173   1120   173
    Lada 1300        1294     68   404   161    955   140
    


```python
#chargement des obs. supplémentaires
IndSup = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="DATA_ACP_IND_SUP",index_col=0)
print(IndSup)
```

                    CYL  PUISS  LONG  LARG  POIDS  VMAX
    Modele                                             
    Peugeot 604    2664    136   472   177   1410   180
    Peugeot 304 S  1288     74   414   157    915   160
    


```python
#coordonnées factorielles
coordSup = acp.transform(IndSup.values)
print(coordSup)
```

    [[ 5.56329226 -0.33860928  0.46428878 -0.40214608  0.38981076  0.08102064]
     [-2.21224139 -1.25777905  0.09304388  0.35370189 -0.648528   -0.12473042]]
    


```python
#position des points supplémentaires dans le plan
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(acp.row_coord_[:,0],acp.row_coord_[:,1],'wo')
ax.axis([-6.5,+6.5,-6.5,+6.5])
ax.plot([-6.5,+6.5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-6.5,+6.5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des individus")

#points actifs
for i in range(n):
    ax.text(acp.row_coord_[i,0],acp.row_coord_[i,1],D.index[i],color="dimgray")
    
#points illutratifs
for i in range(IndSup.shape[0]):
    ax.text(coordSup[i,0],coordSup[i,1],IndSup.index[i],color="blue")
    
plt.show()
```


![png](output_32_0.png)



```python
#nombre de facteurs à prendre en compte
q = 2

#quantile de la loi de Fisher à 95%
pf = stats.f.ppf(0.95,q,n-q)
print(pf)

#seuil
c = (q*(n**2-1))/(n*(n-q))*pf
print(c)

#étendue en abscisse
e1 = 2*math.sqrt(acp.eig_[0][0]*c)
print(e1)

#étendue en ordonnée
e2 = 2*math.sqrt(acp.eig_[0][1]*c)
print(e2)
```

    3.63372346759163
    8.150643611334003
    12.005471835819806
    5.282975914670268
    


```python
#classe pour ellipse
from matplotlib.patches import Ellipse

#position des points supplémentaires dans le plan
#rajouter l'ellipse de Hotelling
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(acp.row_coord_[:,0],acp.row_coord_[:,1],'wo')
ax.axis([-6.5,+6.5,-6.5,+6.5])
ax.plot([-6.5,+6.5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-6.5,+6.5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Ellipse de Hotelling")

#points actifs
for i in range(n):
    ax.text(acp.row_coord_[i,0],acp.row_coord_[i,1],D.index[i],color="dimgray")

#points illutratifs
for i in range(IndSup.shape[0]):
    ax.text(coordSup[i,0],coordSup[i,1],IndSup.index[i],color="blue")
    
#ellispe
ellipse = Ellipse((0,0),width=e1,height=e2,facecolor='none',edgecolor='tomato',linestyle='--')
ax.add_patch(ellipse)
    
plt.show()
```


![png](output_34_0.png)



```python
#variables illustratives
varIllus = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="DATA_ACP_VAR_ILLUS",index_col=0)
print(varIllus)
```

                    FINITION   PRIX  RPOIDPUIS
    Modele                                    
    Alfasud TI           2_B  30570  11.012658
    Audi 100            3_TB  39990  13.058824
    Simca 1300           1_M  29600  15.441176
    Citroen GS Club      1_M  28250  15.762712
    Fiat 132             2_B  34900  11.275510
    Lancia Beta         3_TB  35480  13.170732
    Peugeot 504          2_B  32300  14.683544
    Renault 16 TL        2_B  32000  18.363636
    Renault 30          3_TB  47700  10.312500
    Toyota Corolla       1_M  26540  14.818182
    Alfetta-1.66        3_TB  42395   9.724771
    Princess-1800        2_B  33990  14.146341
    Datsun-200L         3_TB  43980  11.913043
    Taunus-2000          2_B  35010  11.020408
    Rancho              3_TB  39450  14.112500
    Mazda-9295           1_M  27900  13.192771
    Opel-Rekord          2_B  32700  11.200000
    Lada-1300            1_M  22100  14.044118
    


```python
#corrélation de PRIX avec les 2 premiers facteurs
cPrix = numpy.corrcoef(varIllus.PRIX,acp.row_coord_[:,:2],rowvar=False)[0,1:]
print(cPrix)

#corrélation de RPOIDPUIS avec les 2 premiers facteurs
cRPP = numpy.corrcoef(varIllus.RPOIDPUIS,acp.row_coord_[:,:2],rowvar=False)[0,1:]
print(cRPP)
```

    [ 0.77247524 -0.08670844]
    [-0.58903888  0.67254512]
    


```python
#placement dans le cercle des corrélations
#position des points supplémentaires dans le plan
#rajouter l'ellipse de Hotelling
fig, ax = plt.subplots(figsize=(5,5))
ax.axis([-1,+1,-1,+1])
ax.plot([-1,+1],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-1,+1],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Cercle des corrélations")

#variables actives
for i in range(p):
    ax.text(acp.col_coord_[i,0],acp.col_coord_[i,1],D.columns[i],color="dimgray")

#variables illustratives
ax.text(cPrix[0],cPrix[1],'PRIX',color='brown')
ax.arrow(0,0,cPrix[0]-0.1,cPrix[1]+0.04,color='r',head_width=0.05)

ax.text(cRPP[0],cRPP[1],'RPOIDPUIS',color="red")
ax.arrow(0,0,cRPP[0]+0.2,cRPP[1]-0.1,color='r',head_width=0.05)

#cercle
ellipse = Ellipse((0,0),width=2,height=2,facecolor='none',edgecolor='silver',linestyle='--')
ax.add_patch(ellipse)
    
plt.show()
```


![png](output_37_0.png)



```python
#corrélation de prix avec l'ensemble des facteurs
cPrixBis = numpy.corrcoef(varIllus.PRIX,acp.row_coord_[:,:],rowvar=False)[0,1:]
print(cPrixBis)

#au carré : COS2
print(cPrixBis**2)

#somme
print(numpy.sum(cPrixBis**2))
```

    [ 0.77247524 -0.08670844  0.13389277  0.22582891  0.15944978  0.10254878]
    [0.596718   0.00751835 0.01792727 0.0509987  0.02542423 0.01051625]
    0.7091028079636765
    


```python
#data frame temporaire
df = varIllus.copy()
df['F1'] = acp.row_coord_[:,0]
df['F2'] = acp.row_coord_[:,1]
print(df.info())
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 18 entries, Alfasud TI to Lada-1300
    Data columns (total 5 columns):
     #   Column     Non-Null Count  Dtype  
    ---  ------     --------------  -----  
     0   FINITION   18 non-null     object 
     1   PRIX       18 non-null     int64  
     2   RPOIDPUIS  18 non-null     float64
     3   F1         18 non-null     float64
     4   F2         18 non-null     float64
    dtypes: float64(3), int64(1), object(1)
    memory usage: 864.0+ bytes
    None
    


```python
#effectif par groupe de FINITION
n_g = pandas.pivot_table(df,values='F1',index='FINITION',aggfunc='count')
print(n_g)

#moyennes conditionnelles pour le 1er facteur
m_1 = pandas.pivot_table(df,values='F1',index='FINITION',aggfunc='mean')
print(m_1)

#moyennes conditionnelles pour le 2nd facteur
m_2 = pandas.pivot_table(df,values='F2',index='FINITION',aggfunc='mean')
print(m_2)
```

              F1
    FINITION    
    1_M        5
    2_B        7
    3_TB       6
                    F1
    FINITION          
    1_M      -2.000355
    2_B       0.235313
    3_TB      1.392430
                    F2
    FINITION          
    1_M       0.022579
    2_B      -0.045271
    3_TB      0.034001
    


```python
#représentation graphique avec les positions
#position des points supplémentaires dans le plan
fig, ax = plt.subplots(figsize=(10,10))
ax.plot(acp.row_coord_[:,0],acp.row_coord_[:,1],'wo')
ax.axis([-5,+5,-5,+5])
ax.plot([-5,+5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-5,+5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des individus - Modalités de FINITION")

#points actifs
for i in range(n):
    ax.text(acp.row_coord_[i,0],acp.row_coord_[i,1],D.index[i],color="gray")
    
#position des modalités
for g in range(len(m_1.index)):
    ax.text(m_1.F1[g],m_2.F2[g],m_1.index[g],color="red",weight="bold",fontsize=16)
    
plt.show()
```


![png](output_41_0.png)



```python
#rapport de corrélation sur le 1er facteur
print(numpy.sum((n_g.F1/n)*(m_1.F1**2))/acp.eig_[0][0])

#rapport de corrélation sur le 2nd facteur
print(numpy.sum((n_g.F1/n)*(m_2.F2**2))/acp.eig_[0][1])
```

    0.40248443698930525
    0.0015465958518609553
    


```python
#valeurs test sur le 1er facteur
vt_1 = (m_1.F1)/numpy.sqrt((n-n_g.F1)/(n-1)*(acp.eig_[0][0]/n_g.F1))
print(vt_1)
```

    FINITION
    1_M    -2.432717
    2_B     0.368103
    3_TB    1.930766
    Name: F1, dtype: float64
    


```python
#transformer Z en data frame
dfZ = pandas.DataFrame(Z,columns=D.columns,index=varIllus.index)

#rajouter la colonne FINITION
dfZ['FINITION'] = varIllus['FINITION']

#vérification
print(dfZ.info())
```

    <class 'pandas.core.frame.DataFrame'>
    Index: 18 entries, Alfasud TI to Lada-1300
    Data columns (total 7 columns):
     #   Column    Non-Null Count  Dtype  
    ---  ------    --------------  -----  
     0   CYL       18 non-null     float64
     1   PUISS     18 non-null     float64
     2   LONG      18 non-null     float64
     3   LARG      18 non-null     float64
     4   POIDS     18 non-null     float64
     5   VMAX      18 non-null     float64
     6   FINITION  18 non-null     object 
    dtypes: float64(6), object(1)
    memory usage: 1.8+ KB
    None
    


```python
#moyennes conditionnelles sur les variables Z
mz_g = pandas.pivot_table(dfZ,index='FINITION',aggfunc='mean')
print(mz_g)
```

                   CYL      LARG      LONG     POIDS     PUISS      VMAX
    FINITION                                                            
    1_M      -0.777851 -0.826236 -0.823850 -0.825200 -0.909552 -0.735509
    2_B       0.253299  0.147542  0.083116 -0.050267 -0.009218  0.170188
    3_TB      0.352693  0.516398  0.589573  0.746311  0.768714  0.414371
    


```python
#distance à l'origine
disto2_g = numpy.apply_along_axis(arr=mz_g.values,axis=1,func1d=lambda x:numpy.sum(x**2))
print(numpy.sqrt(disto2_g))
```

    [2.00391116 0.35272284 1.43466388]
    


```python
#COS2 sur le premier facteur
print((m_1.values[:,0]**2)/disto2_g)
```

    [0.99645377 0.44506654 0.94199086]
    


```python
#COS2 sur le second facteur
print((m_2.values[:,0]**2)/disto2_g)
```

    [0.00012696 0.01647317 0.00056166]
    
