```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement des données - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="AFC_ETUDES",index_col=0)

#affichage des données
print(D)
```

                    Droit  Sciences  Medecine  IUT
    CSP_vs_Filiere                                
    ExpAgri            80        99        65   58
    Patron            168       137       208   62
    CadreSup          470       400       876   79
    Employe           145       133       135   54
    Ouvrier           166       193       127  129
    


```python
#librairie
import numpy

#calcul des totaux en ligne
tot_lig = numpy.sum(D.values,axis=1) 
print(tot_lig)

#calcul des totaux en colonne
tot_col = numpy.sum(D.values,axis=0) 
print(tot_col)
```

    [ 302  575 1825  467  615]
    [1029  962 1411  382]
    


```python
#profils lignes
prof_lig = numpy.apply_along_axis(arr=D.values,axis=1,func1d=lambda x:x/numpy.sum(x))
print(prof_lig)

#représentation graphique
import matplotlib.pyplot as plt
somme = numpy.zeros(shape=(prof_lig.shape[0]))
for i in range(prof_lig.shape[1]):
    plt.barh(range(prof_lig.shape[0]),prof_lig[:,i],left=somme)
    somme = somme + prof_lig[:,i]
        
plt.yticks(range(prof_lig.shape[0]),D.index)
plt.show()
```

    [[0.26490066 0.32781457 0.21523179 0.19205298]
     [0.29217391 0.23826087 0.36173913 0.10782609]
     [0.25753425 0.21917808 0.48       0.04328767]
     [0.31049251 0.28479657 0.28907923 0.11563169]
     [0.2699187  0.31382114 0.20650407 0.2097561 ]]
    


![png](output_3_1.png)



```python
#profil marginal corresp.
prof_marg_lig = tot_col/numpy.sum(tot_col)
print(prof_marg_lig)
```

    [0.27193446 0.25422833 0.37288584 0.10095137]
    


```python
#distance du KHI-2 entre cadre(2) et ouvrier(4)
print(numpy.sum((prof_lig[2,:]-prof_lig[4,:])**2/prof_marg_lig))

#distance du KHI-2 entre cadre(2) et patron(1)
print(numpy.sum((prof_lig[2,:]-prof_lig[1,:])**2/prof_marg_lig))
```

    0.510900786777672
    0.08461088232967051
    


```python
#distance entre paires de modalités lignes
distPairesLig = numpy.zeros(shape=(prof_lig.shape[0],prof_lig.shape[0]))

#double boucle
for i in range(prof_lig.shape[0]-1):
    for j in range(i+1,prof_lig.shape[0]):
        distPairesLig[i,j] = numpy.sum((prof_lig[i,:]-prof_lig[j,:])**2/prof_marg_lig)
        
#affichage
print(pandas.DataFrame(distPairesLig,index=D.index,columns=D.index))
```

    CSP_vs_Filiere  ExpAgri    Patron  CadreSup   Employe   Ouvrier
    CSP_vs_Filiere                                                 
    ExpAgri             0.0  0.162117  0.453847  0.087400  0.004172
    Patron              0.0  0.000000  0.084611  0.024514  0.191823
    CadreSup            0.0  0.000000  0.000000  0.176847  0.510901
    Employe             0.0  0.000000  0.000000  0.000000  0.115413
    Ouvrier             0.0  0.000000  0.000000  0.000000  0.000000
    


```python
#affichage sous forme de heatmap
import seaborn as sns
sns.heatmap(distPairesLig,vmin=0,vmax=numpy.max(distPairesLig),linewidth=0.1,cmap='Blues',xticklabels=D.index,yticklabels=D.index)
```




    <matplotlib.axes._subplots.AxesSubplot at 0x1c8bddf52c8>




![png](output_7_1.png)



```python
#distance à l'origine
distoLig = numpy.apply_along_axis(arr=prof_lig,axis=1,func1d=lambda x:numpy.sum((x-prof_marg_lig)**2/prof_marg_lig))

#affichage
print(pandas.DataFrame(distoLig,index=D.index))
```

                           0
    CSP_vs_Filiere          
    ExpAgri         0.170350
    Patron          0.003311
    CadreSup        0.069302
    Employe         0.030113
    Ouvrier         0.205492
    


```python
#poids des lignes
poidsLig = tot_lig/numpy.sum(tot_lig)

#inertie des lignes
inertieLig = distoLig * poidsLig

#affichage
print(pandas.DataFrame(numpy.transpose([distoLig,poidsLig,inertieLig]),columns=['Disto2','Poids','Inertie'],index=D.index))
```

                      Disto2     Poids   Inertie
    CSP_vs_Filiere                              
    ExpAgri         0.170350  0.079810  0.013596
    Patron          0.003311  0.151956  0.000503
    CadreSup        0.069302  0.482294  0.033424
    Employe         0.030113  0.123414  0.003716
    Ouvrier         0.205492  0.162526  0.033398
    


```python
#total inertie
tot_InertieLig = numpy.sum(inertieLig)
print(tot_InertieLig)
```

    0.08463685828547157
    


```python
#importation de la librairie
from fanalysis.ca import CA

#lancer les calculs
afc = CA(row_labels=D.index,col_labels=D.columns)
afc.fit(D.values)
```




    CA(col_labels=Index(['Droit', 'Sciences', 'Medecine', 'IUT'], dtype='object'),
       n_components=None,
       row_labels=Index(['ExpAgri', 'Patron', 'CadreSup', 'Employe', 'Ouvrier'], dtype='object', name='CSP_vs_Filiere'),
       stats=True)




```python
#propriétés
print(dir(afc))
```

    ['__class__', '__delattr__', '__dict__', '__dir__', '__doc__', '__eq__', '__format__', '__ge__', '__getattribute__', '__getstate__', '__gt__', '__hash__', '__init__', '__init_subclass__', '__le__', '__lt__', '__module__', '__ne__', '__new__', '__reduce__', '__reduce_ex__', '__repr__', '__setattr__', '__setstate__', '__sizeof__', '__str__', '__subclasshook__', '__weakref__', '_compute_stats', '_compute_svd', '_get_param_names', '_get_tags', '_more_tags', 'c_', 'col_contrib_', 'col_coord_', 'col_cos2_', 'col_labels', 'col_labels_', 'col_labels_short_', 'col_topandas', 'eig_', 'fit', 'fit_transform', 'get_params', 'mapping', 'mapping_col', 'mapping_row', 'model_', 'n_', 'n_components', 'n_components_', 'plot_col_contrib', 'plot_col_cos2', 'plot_eigenvalues', 'plot_row_contrib', 'plot_row_cos2', 'r_', 'row_contrib_', 'row_coord_', 'row_cos2_', 'row_labels', 'row_labels_', 'row_topandas', 'set_params', 'stats', 'transform']
    


```python
#information restituée sur les facteurs
print(afc.eig_)
```

    [[8.23936026e-02 1.70344867e-03 5.39807038e-04]
     [9.73495522e+01 2.01265584e+00 6.37791913e-01]
     [9.73495522e+01 9.93622081e+01 1.00000000e+02]]
    


```python
#coordonnées des modalités lignes
print(pandas.DataFrame(afc.row_coord_,index=D.index))
```

                           0         1         2
    CSP_vs_Filiere                              
    ExpAgri         0.410115 -0.026253  0.038284
    Patron          0.020151  0.026585 -0.046881
    CadreSup       -0.262717 -0.015596  0.006199
    Employe         0.142090  0.097326  0.021242
    Ouvrier         0.451481 -0.039588 -0.009493
    


```python
#vérification de la moyenne pondérée des modalités - 1er facteur
print(numpy.sum(poidsLig * afc.row_coord_[:,0]))

#variance des modalités - 1er facteur
print(numpy.sum(poidsLig * afc.row_coord_[:,0]**2))
```

    6.938893903907228e-17
    0.0823936025778933
    


```python
#affichage dans le premier plan factoriel
fig, ax = plt.subplots(figsize=(10,10))
ax.axis([-0.5,+0.5,-0.5,+0.5])
ax.plot([-0.5,+0.5],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-0.5,+0.5],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des modalités lignes")

for i in range(D.shape[0]):
    ax.text(afc.row_coord_[i,0],afc.row_coord_[i,1],D.index[i])
    
plt.show()
```


![png](output_16_0.png)



```python
#distances euclidienne dans le 1er plan
distPairesLigF1 = numpy.zeros(shape=(prof_lig.shape[0],prof_lig.shape[0]))

#double boucle
for i in range(prof_lig.shape[0]-1):
    for j in range(i+1,prof_lig.shape[0]):
        distPairesLigF1[i,j] = numpy.sum((afc.row_coord_[i,0]-afc.row_coord_[j,0])**2)
        
#affichage
print(pandas.DataFrame(distPairesLigF1,index=D.index,columns=D.index))
```

    CSP_vs_Filiere  ExpAgri    Patron  CadreSup   Employe   Ouvrier
    CSP_vs_Filiere                                                 
    ExpAgri             0.0  0.152072  0.452704  0.071837  0.001711
    Patron              0.0  0.000000  0.080014  0.014869  0.186046
    CadreSup            0.0  0.000000  0.000000  0.163869  0.510079
    Employe             0.0  0.000000  0.000000  0.000000  0.095723
    Ouvrier             0.0  0.000000  0.000000  0.000000  0.000000
    


```python
#profil marginal des filières
prof_marg_col = tot_lig/numpy.sum(tot_lig)
print(prof_marg_col)
```

    [0.07980973 0.1519556  0.48229387 0.12341438 0.16252643]
    


```python
#tableau des profils colonnes
prof_col = numpy.apply_along_axis(arr=D.values,axis=0,func1d=lambda x:x/numpy.sum(x))
print(pandas.DataFrame(prof_col,index=D.index,columns=D.columns))
```

                       Droit  Sciences  Medecine       IUT
    CSP_vs_Filiere                                        
    ExpAgri         0.077745  0.102911  0.046067  0.151832
    Patron          0.163265  0.142412  0.147413  0.162304
    CadreSup        0.456754  0.415800  0.620836  0.206806
    Employe         0.140914  0.138254  0.095677  0.141361
    Ouvrier         0.161322  0.200624  0.090007  0.337696
    


```python
#distance**2 à l'orgine
distoCol = numpy.apply_along_axis(arr=prof_col,axis=0,func1d=lambda x:numpy.sum((x-prof_marg_col)**2/prof_marg_col))

#affichage
print(pandas.DataFrame(distoCol,index=D.columns))
```

                     0
    Droit     0.004738
    Sciences  0.027168
    Medecine  0.092792
    IUT       0.414466
    


```python
#poids de chaque colonne
poidsCol = tot_col/numpy.sum(tot_col)
print(pandas.DataFrame(poidsCol,index=D.columns))
```

                     0
    Droit     0.271934
    Sciences  0.254228
    Medecine  0.372886
    IUT       0.100951
    


```python
#inertie
inertieCol = distoCol*poidsCol
print(pandas.DataFrame(inertieCol,index=D.columns))
```

                     0
    Droit     0.001288
    Sciences  0.006907
    Medecine  0.034601
    IUT       0.041841
    


```python
#somme des inerties
print(numpy.sum(inertieCol))
```

    0.08463685828547156
    


```python
#coordonnées des filières
print(pandas.DataFrame(afc.col_coord_,index=D.columns))
```

                     0         1         2
    Droit     0.027987  0.060669 -0.016545
    Sciences  0.160462  0.002734  0.037583
    Medecine -0.303125 -0.029662 -0.005200
    IUT       0.640174 -0.060749 -0.030870
    


```python
#affichage dans le premier plan factoriel
fig, ax = plt.subplots(figsize=(10,10))
ax.axis([-0.7,+0.7,-0.7,+0.7])
ax.plot([-0.7,+0.7],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-0.7,+0.7],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des modalités colonnes")

for i in range(D.shape[1]):
    ax.text(afc.col_coord_[i,0],afc.col_coord_[i,1],D.columns[i])
    
plt.show()
```


![png](output_25_0.png)



```python
#effectifs totaux
n = numpy.sum(D.values)

#tableau sous indépendance
E = numpy.dot(numpy.reshape(tot_lig,(5,1)),numpy.reshape(tot_col,(1,4)))/n
print(E)
```

    [[ 82.12420719  76.7769556  112.6115222   30.48731501]
     [156.36231501 146.18128964 214.40935518  58.04704017]
     [496.28039112 463.9667019  680.51664905 184.23625793]
     [126.99339323 118.72463002 174.13768499  47.14429175]
     [167.23969345 156.35042283 229.32478858  62.08509514]]
    


```python
#statistique du KHI-2
KHI2 = numpy.sum(((D.values-E)**2)/E)
print(KHI2)

#degré de liberté
ddl = (E.shape[0]-1)*(E.shape[1]-1)
print(ddl)

#librairie scipy pour calcul des CDF
import scipy

#p-value du test
print(1-scipy.stats.chi2.cdf(KHI2,ddl))
```

    320.2658717522244
    12
    0.0
    


```python
#tableau des indices d'attraction et répulsion
IAR = D.values/E

#stat. de test
G = 2*numpy.sum(D.values*numpy.log(IAR))
print(G)

#degré de liberté
print(ddl)

#p-value
print(1-scipy.stats.chi2.cdf(G,ddl))
```

    321.96365868622877
    12
    0.0
    


```python
#représentation simultanée
fig, ax = plt.subplots(figsize=(10,10))
ax.axis([-0.7,+0.7,-0.7,+0.7])
ax.plot([-0.7,+0.7],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-0.7,+0.7],color='silver',linestyle='--')
ax.set_xlabel("Dim.1")
ax.set_ylabel("Dim.2")
plt.title("Carte des modalités lignes et colonnes")

#modalités ligne
for i in range(D.shape[0]):
    ax.text(afc.row_coord_[i,0],afc.row_coord_[i,1],D.index[i],color='blue')

#modalités colonne
for i in range(D.shape[1]):
    ax.text(afc.col_coord_[i,0],afc.col_coord_[i,1],D.columns[i],color='green')
    
plt.show()
```


![png](output_29_0.png)



```python
#pour rappel - les indices observés
print(pandas.DataFrame(IAR,index=D.index,columns=D.columns))
```

                       Droit  Sciences  Medecine       IUT
    CSP_vs_Filiere                                        
    ExpAgri         0.974134  1.289449  0.577206  1.902431
    Patron          1.074428  0.937192  0.970107  1.068099
    CadreSup        0.947045  0.862131  1.287257  0.428797
    Employe         1.141792  1.120239  0.775249  1.145420
    Ouvrier         0.992587  1.234407  0.553800  2.077793
    


```python
#approximation des indices d'attraction-répulsion
estIAR = 1+numpy.dot(numpy.reshape(afc.row_coord_[:,0],(D.shape[0],1)),numpy.reshape(afc.col_coord_[:,0],(1,D.shape[1])))/numpy.sqrt(afc.eig_[0][0])
print(pandas.DataFrame(estIAR,index=D.index,columns=D.columns))
```

                       Droit  Sciences  Medecine       IUT
    CSP_vs_Filiere                                        
    ExpAgri         1.039987  1.229261  0.566907  1.914655
    Patron          1.001965  1.011265  0.978720  1.044941
    CadreSup        0.974385  0.853137  1.277436  0.414078
    Employe         1.013854  1.079431  0.849949  1.316895
    Ouvrier         1.044020  1.252385  0.523224  2.006911
    


```python
#effectifs sous H0
print(E)
```

    [[ 82.12420719  76.7769556  112.6115222   30.48731501]
     [156.36231501 146.18128964 214.40935518  58.04704017]
     [496.28039112 463.9667019  680.51664905 184.23625793]
     [126.99339323 118.72463002 174.13768499  47.14429175]
     [167.23969345 156.35042283 229.32478858  62.08509514]]
    


```python
#coordonnées des modalités lignes dans le plan
print(pandas.DataFrame(afc.row_coord_[:,:2],index=D.index))
```

                           0         1
    CSP_vs_Filiere                    
    ExpAgri         0.410115 -0.026253
    Patron          0.020151  0.026585
    CadreSup       -0.262717 -0.015596
    Employe         0.142090  0.097326
    Ouvrier         0.451481 -0.039588
    


```python
#coordonnées des modalités colonnes dans le plan
print(pandas.DataFrame(afc.col_coord_[:,:2],index=D.columns))
```

                     0         1
    Droit     0.027987  0.060669
    Sciences  0.160462  0.002734
    Medecine -0.303125 -0.029662
    IUT       0.640174 -0.060749
    


```python
#les valeurs propres des 2 premiers facteurs
print(afc.eig_[0][:2])
```

    [0.0823936  0.00170345]
    


```python
#quantité totale d'information -- KHI2 total
print(KHI2)
```

    320.2658717522244
    


```python
#produit des coordonnées croisés dans le plan
#axe 1
s1 = numpy.dot(numpy.reshape(afc.row_coord_[:,0],(5,1)),numpy.reshape(afc.col_coord_[:,0],(1,4)))/numpy.sqrt(afc.eig_[0][0])
#axe 2
s2 = numpy.dot(numpy.reshape(afc.row_coord_[:,1],(5,1)),numpy.reshape(afc.col_coord_[:,1],(1,4)))/numpy.sqrt(afc.eig_[0][1])

#indice d'attraction répulsion dans le plan
IAR_plan = 1+(s1+s2)
print(IAR_plan)
```

    [[1.00139614 1.22752211 0.58577439 1.95329696]
     [1.04104396 1.01302582 0.95961396 1.00581056]
     [0.95145948 0.85210367 1.28864471 0.43703345]
     [1.15691823 1.08587835 0.7800031  1.17364341]
     [0.98582719 1.24976275 0.5516749  2.06518005]]
    


```python
#effectifs estimés dans le plan
est_n_plan = E*IAR_plan
print(est_n_plan)
```

    [[ 82.23886415  94.24541071  65.96494546  59.55077968]
     [162.7800432  148.08542057 205.75021025  58.38432598]
     [472.19068402 395.34772898 876.94417993  80.51740706]
     [146.9209723  128.92050534 135.82793478  55.33058757]
     [164.86943632 195.4009344  126.51272957 128.2168997 ]]
    


```python
#KHI2 dans le premier plan factoriel
KHI2plan = numpy.sum((est_n_plan-E)**2/E)
print(KHI2plan)
```

    318.2232419212952
    


```python
#KHI2 résiduel
KHI2residuel = numpy.sum((D.values-est_n_plan)**2/E)
print(KHI2residuel)
```

    2.0426298309291298
    


```python
#addition des deux = KHI2 total
print(KHI2plan+KHI2residuel)
```

    320.2658717522243
    


```python
#matrice des résidus standardisé
R = (D.values - E)/numpy.sqrt(E)
print(R)
```

    [[-0.23440198  2.53622693 -4.48663409  4.98279921]
     [ 0.93068037 -0.75937762 -0.43771605  0.5188387 ]
     [-1.179691   -2.96968614  7.49359811 -7.7531439 ]
     [ 1.59786954  1.31013775 -2.96584705  0.99847632]
     [-0.09586159  2.93102443 -6.75702234  8.49237551]]
    


```python
#matrice M
M = R/numpy.sqrt(n)
print(M)
```

    [[-0.00381053  0.04122993 -0.07293653  0.08100239]
     [ 0.01512951 -0.01234475 -0.00711569  0.00843445]
     [-0.01917753 -0.04827641  0.12181894 -0.12603823]
     [ 0.02597561  0.02129813 -0.048214    0.01623163]
     [-0.00155836  0.04764791 -0.10984487  0.13805547]]
    


```python
#décomposition en valeurs singulières
sol = numpy.linalg.svd(M)

#valeurs singulières
print(sol[1])

#passés au carré
numpy.set_printoptions(suppress=True)
print(sol[1]**2)
```

    [2.87042858e-01 4.12728563e-02 2.32337478e-02 1.31134238e-17]
    [0.0823936  0.00170345 0.00053981 0.        ]
    


```python
#valeurs propres de l'objet fanalysis
print(afc.eig_[0])
```

    [0.0823936  0.00170345 0.00053981]
    


```python
#vecteurs singuliers à gauche - les facteurs en colonne
print(sol[0])
```

    [[ 0.40363357 -0.17969877  0.46550401 -0.75304822 -0.14495751]
     [ 0.02736552  0.25109394 -0.78656052 -0.55264881  0.11002184]
     [-0.63561972 -0.2624212   0.18528807 -0.28686474  0.64070246]
     [ 0.17390042  0.82841073  0.32118986  0.01236585  0.42447434]
     [ 0.63409564 -0.38669233 -0.1647239   0.21225781  0.61335504]]
    


```python
#coordonnées sur les 3 premiers facteurs
F = numpy.apply_along_axis(arr=sol[0][:,:3]*sol[1][:3],axis=0,func1d=lambda x:x/numpy.sqrt(poidsLig))
print(pandas.DataFrame(F,index=D.index))
```

                           0         1         2
    CSP_vs_Filiere                              
    ExpAgri         0.410115 -0.026253  0.038284
    Patron          0.020151  0.026585 -0.046881
    CadreSup       -0.262717 -0.015596  0.006199
    Employe         0.142090  0.097326  0.021242
    Ouvrier         0.451481 -0.039588 -0.009493
    


```python
#coordonnées des modalités lignes -- objet "afc"
print(pandas.DataFrame(afc.row_coord_,index=D.index))
```

                           0         1         2
    CSP_vs_Filiere                              
    ExpAgri         0.410115 -0.026253  0.038284
    Patron          0.020151  0.026585 -0.046881
    CadreSup       -0.262717 -0.015596  0.006199
    Employe         0.142090  0.097326  0.021242
    Ouvrier         0.451481 -0.039588 -0.009493
    


```python
#vecteurs singuliers à droite - les facteurs en ligne
print(sol[2])
```

    [[ 0.05084468  0.28186205 -0.64485656  0.70861035]
     [ 0.76654132  0.03340234 -0.43885508 -0.46765896]
     [-0.37134184  0.81560391 -0.13667626 -0.4221553 ]
     [ 0.52147336  0.5042106   0.61064379  0.31772846]]
    


```python
#calcul des coordonnées sur les 3 premiers facteurs
G = numpy.apply_along_axis(arr=numpy.transpose(sol[2][:3,:])*sol[1][:3],axis=0,func1d=lambda x:x/numpy.sqrt(poidsCol))
print(pandas.DataFrame(G,index=D.columns))
```

                     0         1         2
    Droit     0.027987  0.060669 -0.016545
    Sciences  0.160462  0.002734  0.037583
    Medecine -0.303125 -0.029662 -0.005200
    IUT       0.640174 -0.060749 -0.030870
    


```python
#coordonnées des modalités colonnes -- objet "afc"
print(pandas.DataFrame(afc.col_coord_,index=D.columns))
```

                     0         1         2
    Droit     0.027987  0.060669 -0.016545
    Sciences  0.160462  0.002734  0.037583
    Medecine -0.303125 -0.029662 -0.005200
    IUT       0.640174 -0.060749 -0.030870
    
