```python
#changement de dossier
import os
os.chdir("D:/_Travaux/university/Cours_Universite/Supports_de_cours/DataAnalysis/Stat/Supports/Analyse_Factorielle/Calculs")
```


```python
#chargement - index_col = 0 pour indiquer que la colonne n°0 est un label
import pandas
D = pandas.read_excel("Data_Methodes_Factorielles.xlsx",sheet_name="MDS_MADAGASCAR",index_col=0)
print(D)

#nombre d'observations
n = D.shape[0]
```

                  Antsiranana  Majunga  Tulear  Fianarantsoa  Tananarive  \
    Ville                                                                  
    Antsiranana             0      859    2038          1522        1109   
    Majunga               859        0    1495           979         566   
    Tulear               2038     1495       0           516         930   
    Fianarantsoa         1522      979     516             0         414   
    Tananarive           1109      566     930           414           0   
    Toamasina            1287      926    1266           750         355   
    
                  Toamasina  
    Ville                    
    Antsiranana        1287  
    Majunga             926  
    Tulear             1266  
    Fianarantsoa        750  
    Tananarive          355  
    Toamasina             0  
    


```python
#librairie numpy
import numpy

#inertie totale
I_tot = numpy.sum(D.values**2)/(2*(n**2))
print(I_tot)
```

    501644.1666666667
    


```python
#importation de la classe de calcul
from sklearn.manifold import MDS

#instanciation
mds = MDS(n_components=2,dissimilarity='precomputed')

#lancement des calculs
mds.fit(D.values)
```




    MDS(dissimilarity='precomputed', eps=0.001, max_iter=300, metric=True,
        n_components=2, n_init=4, n_jobs=None, random_state=None, verbose=0)




```python
#valeur brute du stress (numérateur)
print(mds.stress_)
```

    47646.60256584347
    


```python
#stress ramené à la dispersion initiale (cf. formule en section )
print(numpy.sqrt(mds.stress_/numpy.sum(D.values**2)))
```

    0.036320501623653766
    


```python
#coordonnées
print(pandas.DataFrame(mds.embedding_,index=D.index,columns=['X1','X2']))
```

                          X1           X2
    Ville                                
    Antsiranana   127.541424 -1091.324109
    Majunga       403.445790  -362.895625
    Tulear         56.479148  1007.194885
    Fianarantsoa   -8.496627   474.038401
    Tananarive    -82.638593    32.604814
    Toamasina    -496.331142   -59.618366
    


```python
#est-ce que les axes sont centrés ? => oui
print(numpy.mean(mds.embedding_,axis=0))
```

    [9.47390314e-15 3.78956126e-14]
    


```python
#dispersion sur les 2 axes
I_axes = numpy.mean(mds.embedding_**2,axis=0)
print(I_axes)
```

    [ 72578.52450422 427742.15193544]
    


```python
#dispersion ramenée à l'inertie totale
print(I_axes/I_tot)
```

    [0.14468129 0.85268041]
    


```python
#représentation graphique
#importer la librairie graphique
import matplotlib.pyplot as plt

#préparer le graphique
fig, ax = plt.subplots(figsize=(10,10))
ax.axis([-1100,+1100,-1100,+1100])
ax.plot([-1100,+1100],[0,0],color='silver',linestyle='--')
ax.plot([0,0],[-1100,+1100],color='silver',linestyle='--')
ax.set_xlabel("X1")
ax.set_ylabel("X2")

#ajouter les labels des véhicules
for i in range(n):
    ax.text(-mds.embedding_[i,0],-mds.embedding_[i,1],D.index[i])
    
#faire afficher
plt.show()
```


![png](output_10_0.png)

